# -*- coding: utf-8 -*-

# Check all exceptions under pyxb.ValidationError

# ElementValidationError
# . AbstractElementError
# . ContentInNilInstanceError
# . NoNillableSupportError
# ComplexTypeValidationError
# . AbstractInstantiationError
# . AttributeOnSimpleTypeError
# . ContentValidationError
# - - BatchElementContentError
# - - + IncompleteElementContentError
# - - + UnprocessedElementContentError
# - - IncrementalElementContentError
# - - + UnrecognizedContentError
# - + ExtraSimpleContentError
# - + MixedContentError
# - + SimpleContentAbsentError
# - + UnprocessedKeywordContentError
# AttributeValidationError
# . AttributeChangeError
# . MissingAttributeError
# . ProhibitedAttributeError
# . UnrecognizedAttributeError
# SimpleTypeValueError
# * SimpleFacetValueError
# . SimpleListValueError
# * SimplePluralValueError
# * SimpleUnionValueError

import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)

import pyxb
import pyxb.binding.datatypes as xs
from pyxb.utils import six
import trac26
import unittest
import sys
import xml.dom

# By default skip the "tests" which actually emit the exception
# backtrace.  Sometimes though it's good to see those, since they're
# what the user will normally see first.  Also, this is how we ensure
# that each point where a particular exception might be raised has a
# test case covering it.
DisplayException = False
#DisplayException = True

class TestAbstractElementError (unittest.TestCase):
    Good_xmlt = six.u('<eCardinals><eConcCardCymru>un</eConcCardCymru><eConcCardEnglish>three</eConcCardEnglish></eCardinals>')
    Good_xmld = Good_xmlt.encode('utf-8')

    Bad_xmlt = six.u('<eCardinals><eConcCardCymru>un</eConcCardCymru><eAbstractCard>three</eAbstractCard></eCardinals>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        cym1 = trac26.eConcCardCymru('un')
        eng3 = trac26.eConcCardEnglish('three')

        # Incremental through owning element
        instance = trac26.eCardinals()
        self.assertEqual(0, len(instance.eAbstractCard))
        instance.append(cym1)
        self.assertEqual(1, len(instance.eAbstractCard))
        self.assertTrue(isinstance(instance.eAbstractCard[0], trac26.tCardCymru))
        instance.append(eng3)
        self.assertEqual(2, len(instance.eAbstractCard))
        self.assertTrue(isinstance(instance.eAbstractCard[1], trac26.tCardEnglish))
        self.assertTrue(instance.validateBinding())
        xmld = instance.toxml('utf-8', root_only=True)
        self.assertEqual(xmld, self.Good_xmld)

        # Incremental through construtor element
        instance = trac26.eCardinals(cym1, eng3)
        self.assertEqual(2, len(instance.eAbstractCard))
        self.assertTrue(isinstance(instance.eAbstractCard[0], trac26.tCardCymru))
        self.assertTrue(isinstance(instance.eAbstractCard[1], trac26.tCardEnglish))
        self.assertTrue(instance.validateBinding())
        xmld = instance.toxml('utf-8', root_only=True)
        self.assertEqual(xmld, self.Good_xmld)

        # Through parsing
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(2, len(instance.eAbstractCard))
        self.assertTrue(isinstance(instance.eAbstractCard[0], trac26.tCardCymru))
        self.assertTrue(isinstance(instance.eAbstractCard[1], trac26.tCardEnglish))
        self.assertTrue(instance.validateBinding())

    def testException (self):
        instance = None
        with self.assertRaises(pyxb.AbstractElementError) as cm:
            instance = trac26.eAbstractCard('un')
        e = cm.exception
        self.assertTrue(instance is None)
        self.assertEqual(e.element, trac26.eAbstractCard)
        self.assertEqual(e.value, ('un',))
        self.assertEqual(str(e), 'Cannot instantiate abstract element eAbstractCard directly')

    def testFromDocument (self):
        instance = None
        with self.assertRaises(pyxb.AbstractElementError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(47, e.location.columnNumber)

    def testDisplayException (self):
        if DisplayException:
            trac26.eAbstractCard('un')

    def testIncremental (self):
        # Without type information, incremental does not work.  The content
        # model fails to recognize it, and it looks like mixed content.
        instance = trac26.eCardinals()
        self.assertEqual(0, len(instance.eAbstractCard))
        self.assertRaises(pyxb.MixedContentError, instance.append, 'un')

class TestAbstractInstantiationError (unittest.TestCase):

    Good_xmlt = six.u('<eUseAbstract xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><eAbstract xsi:type="tConcSubCymru"><welsh>un</welsh></eAbstract></eUseAbstract>')
    Good_xmld = Good_xmlt.encode('utf-8')

    Bad_xmlt = six.u('<eUseAbstract xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><eAbstract><welsh>un</welsh></eAbstract></eUseAbstract>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        cym1 = trac26.tConcSubCymru('un')
        eng3 = trac26.tConcSubEnglish('three')
        # Direct generation works
        instance = trac26.eUseAbstract(cym1)
        # So does from documents with xsi:type
        i2 = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertTrue(isinstance(i2.eAbstract, trac26.tConcSubCymru))

    def testException (self):
        scym1 = trac26.tCardCymru('un')
        instance = None
        with self.assertRaises(pyxb.AbstractInstantiationError) as cm:
            instance = trac26.eAbstract(scym1)
        e = cm.exception
        self.assertTrue(instance is None)
        self.assertEqual(str(e), 'Cannot instantiate abstract type tAbstract directly')

    def testFromDocument (self):
        instance = None
        with self.assertRaises(pyxb.AbstractInstantiationError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertTrue(instance is None)
        # Location within Bad_xmlt where error occurred
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(68, e.location.columnNumber)

    def testDisplayException (self):
        if DisplayException:
            scym1 = trac26.tCardCymru('un')
            trac26.eAbstract(scym1)

    def testDisplayExceptionDoc (self):
        if DisplayException:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)

class TestSimpleContentAbsentError (unittest.TestCase):

    Good_xmlt = six.u('<eCTwSC>3</eCTwSC>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eCTwSC></eCTwSC>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    GoodSeq_xmlt = six.u('<eCTwSCSequence><eCTwSC>1</eCTwSC><eCTwSC>2</eCTwSC></eCTwSCSequence>')
    GoodSeq_xmld = GoodSeq_xmlt.encode('utf-8')
    BadSeq_xmlt = six.u('<eCTwSCSequence><eCTwSC>1</eCTwSC><eCTwSC></eCTwSC></eCTwSCSequence>')
    BadSeq_xmld = BadSeq_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.eCTwSC(3)
        self.assertEqual(3, instance.value())
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(3, instance.value())
        instance = trac26.eCTwSCSequence()
        # Can't infer conversion, see trac/175
        instance.eCTwSC.append(trac26.eCTwSC(1))
        instance.eCTwSC.append(trac26.eCTwSC(2))
        instance = trac26.CreateFromDocument(self.GoodSeq_xmlt)
        xmld = instance.toxml('utf-8', root_only=True)
        self.assertEqual(xmld, self.GoodSeq_xmld)

    def testDirect (self):
        instance = None
        with self.assertRaises(pyxb.SimpleContentAbsentError) as cm:
            instance = trac26.eCTwSC()
        e = cm.exception
        self.assertTrue(instance is None)

    def testXsdConstraints (self):
        instance = trac26.eCTwSC(3)
        self.assertTrue(instance.validateBinding())
        instance.reset()
        with self.assertRaises(pyxb.SimpleContentAbsentError) as cm:
            instance.xsdConstraintsOK()
        e = cm.exception
        self.assertEqual(str(e), 'Type tCTwSC requires content')

    def testAfterReset (self):
        instance = trac26.eCTwSC(3)
        self.assertTrue(instance.validateBinding())
        instance.reset()
        with self.assertRaises(pyxb.SimpleContentAbsentError) as cm:
            instance.validateBinding()
        e = cm.exception
        self.assertEqual(str(e), 'Type tCTwSC requires content')

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.SimpleContentAbsentError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertTrue(instance is None)
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(0, e.location.columnNumber)
        self.assertEqual(str(e), 'Type {http://www.w3.org/2001/XMLSchema}int requires content')

    def testDocumentSeq (self):
        instance = None
        with self.assertRaises(pyxb.SimpleContentAbsentError) as cm:
            instance = trac26.CreateFromDocument(self.BadSeq_xmlt)
        e = cm.exception
        self.assertTrue(instance is None)
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(34, e.location.columnNumber)
        self.assertEqual(str(e), 'Type {http://www.w3.org/2001/XMLSchema}int requires content')

    def testDisplayException (self):
        if DisplayException:
            instance = trac26.eCTwSC()

    def testDisplayExceptionDoc (self):
        if DisplayException:
            instance = trac26.CreateFromDocument(self.BadSeq_xmlt)

    def testDisplayExceptionReset (self):
        if DisplayException:
            instance = trac26.eCTwSC(3)
            instance.reset()
            instance.validateBinding()

    def testDisplayExceptionXsdConstraints (self):
        if DisplayException:
            instance = trac26.eCTwSC(3)
            instance.reset()
            instance.xsdConstraintsOK()

class TestAttributeChangeError (unittest.TestCase):

    Good_xmlt = six.u('<eAttributes aFixed="5" aReq="2"/>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eAttributes aFixed="2" aReq="2"/>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.tAttributes(aReq=2)
        self.assertEqual(instance.aReq, 2)
        self.assertEqual(instance.aFixed, 5)
        self.assertTrue(instance.validateBinding())
        # OK to explicitly assign fixed value
        instance.aFixed = 5
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8',root_only=True))

    def testDirect (self):
        instance = trac26.tAttributes(aReq=2)
        with self.assertRaises(pyxb.AttributeChangeError) as cm:
            instance.aFixed = 1
        e = cm.exception
        self.assertEqual(e.type, trac26.tAttributes)
        self.assertEqual(e.tag, 'aFixed')
        self.assertEqual(e.instance, instance)
        self.assertTrue(e.location is None)

    def testDisplayException (self):
        if DisplayException:
            trac26.tAttributes(aReq=2).aFixed = 1

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.AttributeChangeError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        self.assertTrue(instance is None)
        e = cm.exception
        self.assertEqual(e.type, trac26.tAttributes)
        self.assertEqual(e.tag, 'aFixed')
        self.assertFalse(e.instance is None) # there, but partially defined
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(0, e.location.columnNumber)

    def testDisplayExceptionDoc (self):
        if DisplayException:
            trac26.CreateFromDocument(self.Bad_xmlt)

class TestAttributeValueError (unittest.TestCase):
    # AttributeValueError was intended to be raised when batch
    # validation discovered that the value was not suited for the type
    # of the attribute.  This can no longer happen since attribute
    # values are validated when they are assigned.  Consequently such
    # errors show up as SimpleTypeValueErrors.  Test for those instead.

    Good_xmlt = six.u('<eAttributes aCardCymru="pedwar" aReq="4"/>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eAttributes aCardCymru="four" aReq="4"/>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.tAttributes(aCardCymru='pedwar', aReq=4)
        self.assertTrue(instance.validateBinding())
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))
        instance.aCardCymru = 'dau'
        self.assertTrue(instance.validateBinding())

    def testAssignment (self):
        instance = trac26.eAttributes(aReq=4)
        with self.assertRaises(pyxb.SimpleTypeValueError) as cm:
            instance.aCardCymru = 'four'
        e = cm.exception
        au = trac26.tAttributes._AttributeMap['aCardCymru']
        self.assertTrue(isinstance(e, pyxb.SimpleFacetValueError))
        self.assertEqual(e.type, au.dataType())
        self.assertEqual(e.value, 'four')
        self.assertEqual(e.facet, au.dataType()._CF_enumeration)
        self.assertEqual(str(e), "Type tCardCymru enumeration constraint violated by value four")

    def testConstructor (self):
        with self.assertRaises(pyxb.SimpleTypeValueError) as cm:
            instance = trac26.eAttributes(aReq=4, aCardCymru='four')

    def testDocument (self):
        with self.assertRaises(pyxb.SimpleTypeValueError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)

class TestMissingAttributeError (unittest.TestCase):
    Good_xmlt = six.u('<eAttributes aReq="4"/>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eAttributes/>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.tAttributes(aReq=4)
        self.assertTrue(instance.validateBinding())
        instance = trac26.tAttributeReqFixed(aReqFixed=9)
        self.assertTrue(instance.validateBinding())
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))

    def testBatch (self):
        instance = trac26.tAttributes()
        with self.assertRaises(pyxb.MissingAttributeError) as cm:
            instance.validateBinding()

    def testBatchReqFixed (self):
        instance = trac26.tAttributeReqFixed()
        with self.assertRaises(pyxb.MissingAttributeError) as cm:
            instance.validateBinding()

    def testAssignment (self):
        instance = trac26.tAttributes(aReq=3)
        self.assertTrue(instance.validateBinding())
        with self.assertRaises(pyxb.MissingAttributeError) as cm:
            instance.aReq = None
        e = cm.exception
        self.assertEqual(e.instance, instance)

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.MissingAttributeError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        au = trac26.tAttributes._AttributeMap['aReq']
        self.assertFalse(e.instance is None)
        self.assertEqual(e.type, trac26.tAttributes)
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(0, e.location.columnNumber)
        self.assertEqual(str(e), "Instance of <class 'trac26.tAttributes'> lacks required attribute aReq")

    def testDisplayBatch (self):
        instance = trac26.tAttributes()
        if DisplayException:
            instance.validateBinding()

    def testDisplayBatchReqFixed (self):
        instance = trac26.tAttributeReqFixed()
        if DisplayException:
            instance.validateBinding()

    def testDisplayDoc (self):
        if DisplayException:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)

class TestProhibitedAttributeError (unittest.TestCase):
    Good_xmlt = six.u('<eAttributes aProhibited="6" aReq="4"/>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eAttributesProhibited aProhibited="6" aReq="4"/>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        i1 = trac26.eAttributes(aReq=2, aProhibited=6)
        self.assertTrue(i1.validateBinding())
        i2 = trac26.eAttributesProhibited(aReq=2)
        self.assertTrue(i2.validateBinding())
        self.assertTrue(isinstance(i2, type(i1)))
        self.assertFalse(isinstance(i1, type(i2)))
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))

    def testConstructor (self):
        instance = None
        with self.assertRaises(pyxb.ProhibitedAttributeError) as cm:
            instance = trac26.eAttributesProhibited(aReq=2, aProhibited=6)

    def testAssignment (self):
        instance = trac26.eAttributesProhibited(aReq=2)
        with self.assertRaises(pyxb.ProhibitedAttributeError) as cm:
            instance.aProhibited = 6

    def testSet (self):
        instance = trac26.eAttributesProhibited(aReq=2)
        au = instance._AttributeMap['aProhibited']
        with self.assertRaises(pyxb.ProhibitedAttributeError) as cm:
            au.set(instance, 6)

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.ProhibitedAttributeError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertFalse(e.instance is None)
        self.assertEqual(e.type, trac26.tAttributesProhibited)
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(0, e.location.columnNumber)
        self.assertEqual(str(e), "Attempt to reference prohibited attribute aProhibited in type <class 'trac26.tAttributesProhibited'>")

    def testDisplay (self):
        instance = trac26.eAttributesProhibited(aReq=2)
        if DisplayException:
            instance.aProhibited = 6

    def testDisplayDoc (self):
        if DisplayException:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)

class TestUnrecognizedAttributeError (unittest.TestCase):
    Good_xmlt = six.u('<eAttributes aReq="4"/>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eAttributes aReq="4" aBad="1"/>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        dom = pyxb.utils.domutils.StringToDOM(self.Good_xmlt)
        instance = trac26.CreateFromDOM(dom)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))

    def testDOM (self):
        dom = pyxb.utils.domutils.StringToDOM(self.Bad_xmlt)
        with self.assertRaises(pyxb.UnrecognizedAttributeError) as cm:
            instance = trac26.CreateFromDOM(dom)
        e = cm.exception
        # The code path for this is creating a map from attribute tags
        # to values in isolation of the specific instance.  A
        # partially-constructed instance is available, but the
        # location is not available.
        self.assertEqual(e.type, trac26.tAttributes)
        self.assertEqual(e.tag, 'aBad')
        self.assertTrue(e.instance is not None)
        self.assertTrue(isinstance(e.instance, e.type))
        self.assertTrue(e.location is None)

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.UnrecognizedAttributeError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        self.assertTrue(instance is None)
        e = cm.exception
        self.assertEqual(e.type, trac26.tAttributes)
        self.assertEqual(e.tag, 'aBad')
        # In this case we were given an instance, which provides a
        # location.  Note that initialization of the instance was left
        # incomplete.
        self.assertFalse(e.instance is None)
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(0, e.location.columnNumber)

    def testDisplayDOM (self):
        if DisplayException:
            trac26.CreateFromDOM(pyxb.utils.domutils.StringToDOM(self.Bad_xmlt))

    def testDisplayDoc (self):
        if DisplayException:
            trac26.CreateFromDocument(self.Bad_xmlt)

class TestAttributeOnSimpleTypeError (unittest.TestCase):
    Good_xmlt = six.u('<eInts><eInt>1</eInt></eInts>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eInts><eInt bits="1">1</eInt></eInts>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.eInts(1,2)
        self.assertEqual(2, len(instance.eInt))
        e0 = instance.eInt[0]
        self.assertTrue(isinstance(e0, trac26.eInt.typeDefinition()))
        self.assertEqual(e0, 1)
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))

    def testException (self):
        instance = None
        with self.assertRaises(pyxb.AttributeOnSimpleTypeError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertEqual(e.tag, 'bits')
        self.assertEqual(e.value, six.u('1'))
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(7, e.location.columnNumber)
        self.assertEqual(str(e), "Simple type {http://www.w3.org/2001/XMLSchema}int cannot support attribute bits")

    def testDisplay (self):
        if DisplayException:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)

class TestUnprocessedKeywordContentError (unittest.TestCase):
    def testException (self):
        instance = None
        with self.assertRaises(pyxb.UnprocessedKeywordContentError) as cm:
            instance = trac26.eAttributes(foo=3)
        self.assertTrue(instance is None)
        e = cm.exception
        self.assertTrue(isinstance(e.instance, trac26.tAttributes))
        self.assertEqual(1, len(e.keywords))
        self.assertEqual(3, e.keywords['foo'])
        self.assertTrue(e.location is None)
        self.assertEqual(str(e), 'Unprocessed keywords instantiating tAttributes: foo')

class TestExtraSimpleContentError (unittest.TestCase):

    def testSchemaSupport (self):
        instance = trac26.eCTwSC(1)
        self.assertTrue(instance.validateBinding())
        instance.reset()
        self.assertRaises(pyxb.SimpleContentAbsentError, instance.validateBinding)
        instance.append(1)
        self.assertTrue(instance.validateBinding())

    def testException (self):
        instance = trac26.eCTwSC(1)
        with self.assertRaises(pyxb.ExtraSimpleContentError) as cm:
            instance.append(2)
        e = cm.exception
        self.assertEqual(e.instance, instance)
        self.assertEqual(e.value, 2)
        self.assertTrue(e.location is None)
        self.assertEqual(str(e), 'Instance of tCTwSC already has simple content value assigned')

    def testDisplay (self):
        instance = trac26.eCTwSC(1)
        if DisplayException:
            instance.append(2)

class TestMixedContentError (unittest.TestCase):
    Good_xmlt = six.u('<eCTwSCSequence><eCTwSC>2</eCTwSC></eCTwSCSequence>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eCTwSCSequence><eCTwSC>2</eCTwSC>noise</eCTwSCSequence>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.eCTwSCSequence()
        instance.append(trac26.eCTwSC(2))
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))

    def testException (self):
        instance = trac26.eCTwSCSequence()
        instance.append(trac26.eCTwSC(2))
        with self.assertRaises(pyxb.MixedContentError) as cm:
            instance.append('noise')
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertEqual(e.instance, instance)
        self.assertEqual(e.value, 'noise')
        self.assertEqual(str(e), 'Invalid non-element content')

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.MixedContentError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        loctext = ''
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(34, e.location.columnNumber)
            loctext = ' at <unknown>[1:34]'
        self.assertEqual(e.value, 'noise')
        self.assertEqual(str(e), 'Invalid non-element content{}'.format(loctext))

    def testDisplay (self):
        if DisplayException:
            instance = trac26.eCTwSCSequence()
            instance.append(trac26.eCTwSC(2))
            instance.append('noise')

    def testDisplayDoc (self):
        if DisplayException:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)

class TestUnrecognizedContentError (unittest.TestCase):
    Good_xmlt = six.u('<eInts><eInt>1</eInt></eInts>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eInts><eCTwSC>1</eCTwSC></eInts>')
    Bad_details = '''The containing element eIntsPlus is defined at trac26.xsd[80:2].
The containing element type <class 'trac26.CTD_ANON_3'> is defined at trac26.xsd[81:4]
The unrecognized content is eCTwSC
The <class 'trac26.CTD_ANON_3'> automaton is in an accepting state.
The following element and wildcard content would be accepted:
\tAn element eInt per trac26.xsd[83:8]
\tA wildcard per trac26.xsd[84:8]'''
    Bad2_xmlt = six.u('<eInts><something>1</something></eInts>')

    def testSchemaSupport (self):
        instance = trac26.eInts()
        instance.append(1)
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))

    def testException (self):
        instance = trac26.eIntsPlus()
        with self.assertRaises(pyxb.UnrecognizedContentError) as cm:
            instance.append(trac26.eCTwSC(2))
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertTrue(e.automaton_configuration is not None)
        self.assertTrue(isinstance(e.value, trac26.tCTwSC))
        acceptable = e.automaton_configuration.acceptableContent()
        self.assertEqual(2, len(acceptable))
        self.assertTrue(isinstance(acceptable[0], pyxb.binding.content.ElementUse))
        self.assertNotEqual(acceptable[0].elementBinding(), trac26.eInt)
        self.assertEqual(acceptable[0].typeDefinition(), trac26.eInt.typeDefinition())
        self.assertTrue(isinstance(acceptable[1], pyxb.binding.content.WildcardUse))
        self.assertEqual(str(e), 'Invalid content eCTwSC (expect eInt or xs:any)')
        if pyxb.XMLStyle_minidom != pyxb._XMLStyle:
            self.assertEqual(e.details(), self.Bad_details)

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.UnrecognizedContentError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        loctext = ''
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(7, e.location.columnNumber)
            loctext= ' at <unknown>[1:7]'
        self.assertTrue(isinstance(e.value, trac26.tCTwSC))
        self.assertEqual(e._valueDescription(), 'eCTwSC')
        acceptable = e.automaton_configuration.acceptableContent()
        self.assertEqual(1, len(acceptable))
        self.assertNotEqual(acceptable[0].elementBinding(), trac26.eInt)
        self.assertEqual(acceptable[0].typeDefinition(), trac26.eInt.typeDefinition())
        self.assertEqual(str(e), 'Invalid content eCTwSC{} (expect eInt)'.format(loctext))

    def testDocument2 (self):
        instance = None
        with self.assertRaises(pyxb.UnrecognizedContentError) as cm:
            instance = trac26.CreateFromDocument(self.Bad2_xmlt)
        e = cm.exception
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        loctext = ''
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(7, e.location.columnNumber)
            loctext= ' at <unknown>[1:7]'
        self.assertTrue(isinstance(e.value, xml.dom.Node))
        self.assertFalse(isinstance(e.value, pyxb.binding.basis._TypeBinding_mixin))
        self.assertEqual(e._valueDescription(), 'something')
        acceptable = e.automaton_configuration.acceptableContent()
        self.assertEqual(1, len(acceptable))
        self.assertNotEqual(acceptable[0].elementBinding(), trac26.eInt)
        self.assertEqual(acceptable[0].typeDefinition(), trac26.eInt.typeDefinition())
        self.assertEqual(str(e), 'Invalid content something{} (expect eInt)'.format(loctext))

    def testDisplay (self):
        if DisplayException:
            instance = trac26.eInts()
            instance.append(trac26.eCTwSC(2))

    def testDisplayDoc (self):
        if DisplayException:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)

class TestIncompleteElementContentError (unittest.TestCase):
    Good_xmlt = six.u('<eTranslateCard><eConcCardCymru>un</eConcCardCymru><eConcCardEnglish>one</eConcCardEnglish><cardinal>1</cardinal></eTranslateCard>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eTranslateCard><eConcCardEnglish>one</eConcCardEnglish><cardinal>1</cardinal></eTranslateCard>')
    Bad_xmld = Bad_xmlt.encode('utf-8')
    Bad_details = '''The containing element eTranslateCard is defined at trac26.xsd[124:2].
The containing element type tTranslateCard is defined at trac26.xsd[115:2]
The tTranslateCard automaton is not in an accepting state.
The last accepted content was eConcCardCymru
The following element and wildcard content would be accepted:
\tAn element eConcCardEnglish per trac26.xsd[119:8]
The following content was not processed by the automaton:
\tcardinal (1 instances)'''
    Bad_details_noloc = '''The containing element eTranslateCard is defined at None.
The containing element type tTranslateCard is defined at None
The tTranslateCard automaton is not in an accepting state.
The last accepted content was eConcCardCymru
The following element and wildcard content would be accepted:
\tAn element eConcCardEnglish per None
The following content was not processed by the automaton:
\tcardinal (1 instances)'''

    def testSchemaSupport (self):
        instance = trac26.eTranslateCard(trac26.eConcCardCymru('un'),
                                         trac26.eConcCardEnglish('one'),
                                         xs.int(1))
        self.assertTrue(instance.validateBinding())
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))
        instance = trac26.eTranslateCard(trac26.eConcCardCymru('un'))
        instance.cardinal = 1
        instance.eConcCardEnglish = 'one'
        self.assertTrue(instance.validateBinding())

    def testException (self):
        instance = trac26.eTranslateCard(trac26.eConcCardCymru('un'))
        instance.cardinal = 1
        with self.assertRaises(pyxb.IncompleteElementContentError) as cm:
            instance.validateBinding()
        e = cm.exception
        self.assertFalse(e.fac_configuration.isAccepting())
        self.assertEqual(1, len(e.symbols))
        self.assertEqual(1, len(e.symbol_set))
        (ed, syms) = next(six.iteritems(e.symbol_set))
        self.assertEqual(1, len(syms))
        self.assertEqual(instance.cardinal, syms[0])
        expect = self.Bad_details
        if pyxb.XMLStyle_minidom == pyxb._XMLStyle:
            expect = self.Bad_details_noloc
        self.assertEqual(e.details(), expect)

class TestUnprocessedElementContentError (unittest.TestCase):
    Bad_details = '''The containing element eTranslateCardMulti is defined at trac26.xsd[132:2].
The containing element type tTranslateCardMulti is defined at trac26.xsd[125:2]
The tTranslateCardMulti automaton is in an accepting state.
The last accepted content was cardinal
The following element and wildcard content would be accepted:
\tAn element eConcCardCymru per trac26.xsd[127:6]
The following content was not processed by the automaton:
\tcardinal (1 instances)'''

    def testSchemaSupport (self):
        instance = trac26.eTranslateCardMulti(trac26.eConcCardCymru('un'),
                                              trac26.eConcCardEnglish('one'),
                                              xs.int(1))
        self.assertTrue(instance.validateBinding())
        instance.eConcCardCymru.append('dau')
        instance.eConcCardEnglish.append('two')
        instance.cardinal.append(2)
        self.assertTrue(instance.validateBinding())

    def testException (self):
        instance = trac26.eTranslateCardMulti(trac26.eConcCardCymru('un'),
                                              trac26.eConcCardEnglish('one'),
                                              xs.int(1))
        self.assertTrue(instance.validateBinding())
        instance.cardinal.append(2)
        with self.assertRaises(pyxb.UnprocessedElementContentError) as cm:
            instance.validateBinding()
        e = cm.exception
        self.assertTrue(e.fac_configuration.isAccepting())
        self.assertEqual(3, len(e.symbols))
        self.assertEqual(1, len(e.symbol_set))
        (ed, syms) = next(six.iteritems(e.symbol_set))
        self.assertEqual(1, len(syms))
        self.assertEqual(instance.cardinal[1], syms[0])
        if pyxb.XMLStyle_minidom != pyxb._XMLStyle:
            self.assertEqual(e.details(), self.Bad_details)

class TestContentInNilInstanceError (unittest.TestCase):
    Good_xmlt = six.u('<eNilCTwSC xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true"/>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eNilCTwSC xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:nil="true">4</eNilCTwSC>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.eNilCTwSC(4)
        self.assertEqual(instance.value(), 4)
        self.assertTrue(instance.validateBinding())
        instance = trac26.eNilCTwSC(_nil=True)
        self.assertTrue(instance.value() is None)
        self.assertTrue(instance.validateBinding())
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))
        instance = trac26.eNilInt(4)
        self.assertEqual(4, instance)
        instance = trac26.eNilInt(_nil=True)
        self.assertTrue(instance.validateBinding())
        instance = trac26.eNilCTwSC(3)
        instance.reset()
        instance.append(4)
        self.assertTrue(instance.validateBinding())
        instance = trac26.eNilOneInt()
        instance.eInt = 5
        self.assertTrue(instance.validateBinding())
        instance = trac26.eNilInts()
        instance.eInt.append(1)
        self.assertTrue(instance.validateBinding())

    def testConstructor (self):
        with self.assertRaises(pyxb.ContentInNilInstanceError) as cm:
            trac26.eNilInt(4, _nil=True)
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertEqual(e.instance._element(), trac26.eNilInt)
        self.assertEqual(e.content, 4)

    def testAppend (self):
        instance = trac26.eNilCTwSC(_nil=True)
        with self.assertRaises(pyxb.ContentInNilInstanceError) as cm:
            instance.append(4)
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertEqual(e.content, 4)

    def testValidate (self):
        instance = trac26.eNilInts(1, 2, 3)
        self.assertTrue(instance.validateBinding())
        instance._setIsNil(True)
        self.assertTrue(instance.validateBinding())
        # Not a valid way to do things, but only way to test this raise
        instance.content().append(1)
        with self.assertRaises(pyxb.ContentInNilInstanceError) as cm:
            instance.validateBinding()
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertEqual(e.content, [1])

    def testContentAppend (self):
        instance = trac26.eNilInts(_nil=True)
        self.assertTrue(instance._isNil())
        with self.assertRaises(pyxb.ContentInNilInstanceError) as cm:
            type(instance)._UseForTag('eInt').append(instance, 1)
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertEqual(e.instance, instance)

    def testContentSet (self):
        instance = trac26.eNilOneInt(_nil=True)
        with self.assertRaises(pyxb.ContentInNilInstanceError) as cm:
            instance.eInt = 1
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertEqual(e.instance, instance)

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.ContentInNilInstanceError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertFalse(e.location is None)
        self.assertEqual(e.content, six.u('4'))
        self.assertEqual(1, e.location.lineNumber)
        self.assertEqual(0, e.location.columnNumber)
        self.assertEqual(str(e), 'eNilCTwSC with {http://www.w3.org/2001/XMLSchema-instance}nil=true cannot have content')

    def testDisplayDoc (self):
        if DisplayException:
            trac26.CreateFromDocument(self.Bad_xmlt)

class TestContentInNilInstanceError (unittest.TestCase):
    def testSchemaSupport (self):
        instance = trac26.eNilCTwSC(4)
        instance._setIsNil(True)
        self.assertTrue(instance.validateBinding())

    def testException (self):
        instance = trac26.eCTwSC(4)
        with self.assertRaises(pyxb.NoNillableSupportError) as cm:
            instance._setIsNil(True)
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertEqual(e.instance, instance)

class TestSimpleListValueError (unittest.TestCase):
    # Note: Unable to come up with code to test raise in _CheckValidValue;
    # values get validated before that point.
    Good_xmlt = six.u('<eListCardCymru>un dau tri pedwar</eListCardCymru>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eListCardCymru>un dau tri pedwar pump</eListCardCymru>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.eListCardCymru()
        instance.append('un')
        instance.append('dau')
        self.assertTrue(instance.validateBinding())
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))
        instance = trac26.eSimpleInstance()
        self.assertTrue(instance.validateBinding())
        instance.eSimpleInstance = [ 'un', 'dau', 'tri' ]
        self.assertTrue(instance.validateBinding())

    def testException (self):
        instance = trac26.eListCardCymru()
        with self.assertRaises(pyxb.SimpleListValueError) as cm:
            instance.append('pump')
        e = cm.exception
        self.assertEqual(e.type, trac26.tListCardCymru)
        self.assertEqual(e.value, 'pump')
        self.assertTrue(e.location is None)
        self.assertEqual(str(e), 'Member type tCardCymru of list type tListCardCymru cannot accept pump')

    def testException (self):
        instance = None
        with self.assertRaises(pyxb.SimpleListValueError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertEqual(e.type, trac26.tListCardCymru)
        self.assertEqual(e.value, 'pump')
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(0, e.location.columnNumber)

class TestSimpleUnionValueError (unittest.TestCase):
    Good_xmlt = six.u('<eSimpleInstance><eUnionCard>dau</eUnionCard></eSimpleInstance>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eSimpleInstance><eUnionCard>pump</eUnionCard></eSimpleInstance>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        for (ty, va) in ((trac26.tCardCymru, 'un'), (trac26.tCardEnglish, 'two') , (xs.int, 3)):
            instance = trac26.eUnionCard(va)
            self.assertTrue(isinstance(instance, ty))
            xmlt = six.u('<eUnionCard>%s</eUnionCard>') % (va,)
            xmld = xmlt.encode('utf-8')
            self.assertEqual(xmld, instance.toxml('utf-8', root_only=True))
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))

    def testException (self):
        instance = None
        with self.assertRaises(pyxb.SimpleUnionValueError) as cm:
            instance = trac26.eUnionCard('pump')
        e = cm.exception
        self.assertEqual(e.type, trac26.tUnionCard)
        self.assertEqual(e.value, ('pump',))
        self.assertTrue(e.location is None)
        self.assertEqual(str(e), "No memberType of tUnionCard can be constructed from ('pump',)")

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.SimpleUnionValueError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertEqual(e.type, trac26.tUnionCard)
        if pyxb.XMLStyle_saxer == pyxb._XMLStyle:
            self.assertEqual(e.value, ('pump',))
            self.assertFalse(e.location is None)
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(17, e.location.columnNumber)

class TestSimpleFacetValueError (unittest.TestCase):
    Good_xmlt = six.u('<eTLA>ABC</eTLA>')
    Good_xmld = Good_xmlt.encode('utf-8')
    Bad_xmlt = six.u('<eTLA>ABCD</eTLA>')
    Bad_xmld = Bad_xmlt.encode('utf-8')

    def testSchemaSupport (self):
        instance = trac26.eTLA('ABC')
        self.assertTrue(instance.validateBinding())
        instance = trac26.CreateFromDocument(self.Good_xmlt)
        self.assertEqual(self.Good_xmld, instance.toxml('utf-8', root_only=True))

    def testException (self):
        instance = None
        with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
            instance = trac26.eTLA('ABCD')
        e = cm.exception
        self.assertTrue(e.location is None)
        self.assertEqual(e.type, trac26.tTLA)
        self.assertEqual(e.value, 'ABCD')
        self.assertEqual(e.facet, trac26.tTLA._CF_length)
        self.assertEqual(str(e), 'Type tTLA length constraint violated by value ABCD')

    def testDocument (self):
        instance = None
        with self.assertRaises(pyxb.SimpleFacetValueError) as cm:
            instance = trac26.CreateFromDocument(self.Bad_xmlt)
        e = cm.exception
        self.assertFalse((e.location is None) and (pyxb.XMLStyle_saxer == pyxb._XMLStyle))
        if e.location is not None:
            self.assertEqual(1, e.location.lineNumber)
            self.assertEqual(0, e.location.columnNumber)
        self.assertEqual(e.type, trac26.tTLA)
        self.assertEqual(e.value, 'ABCD')
        self.assertEqual(e.facet, trac26.tTLA._CF_length)
        self.assertEqual(str(e), 'Type tTLA length constraint violated by value ABCD')

if __name__ == '__main__':
    if sys.version_info[:2] >= (2, 7):
        unittest.main()
    else:
        _log.warning('Cannot run test prior to Python 2.7')



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
# Undeclared XML namespace
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.utils.domutils
from xml.dom import Node

import os.path


import unittest

class TestIssue0006 (unittest.TestCase):

    local_xsd = '''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="ce">
    <xs:complexType name="cet">
      <xs:sequence>
        <xs:element name="s" type="xs:string"/>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>
'''

    top_xsd = '''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:complexType name="cet">
    <xs:sequence>
      <xs:element name="s" type="xs:string"/>
    </xs:sequence>
  </xs:complexType>
</xs:schema>
'''

    def testNamedLocal (self):
        with self.assertRaises(pyxb.SchemaValidationError) as cm:
            code = pyxb.binding.generate.GeneratePython(schema_text=self.local_xsd)
        e = cm.exception

    def testUnnamedLocal (self):
        fixed = self.local_xsd.replace('name="cet"', '')
        code = pyxb.binding.generate.GeneratePython(schema_text=fixed)

    def testNamedTop (self):
        code = pyxb.binding.generate.GeneratePython(schema_text=self.top_xsd)

    def testUnnamedTop (self):
        broken = self.top_xsd.replace('name="cet"', '')
        with self.assertRaises(pyxb.SchemaValidationError) as cm:
            code = pyxb.binding.generate.GeneratePython(schema_text=broken)
        e = cm.exception

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-simplerange.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestKML (unittest.TestCase):
    def testAngle360 (self):
        self.assertEqual(25.4, angle360(25.4))
        self.assertRaises(SimpleTypeValueError, angle360, 420.0)
        self.assertRaises(SimpleTypeValueError, angle360, -361.0)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xs:complexType name="YesNoChoice">
        <xs:annotation>
            <xs:documentation>Yes No Choice</xs:documentation>
        </xs:annotation>
        <xs:choice>
            <xs:element name="Yes" type="xs:boolean" fixed="true"/>
            <xs:element name="No" type="xs:boolean" fixed="true"/>
        </xs:choice>
    </xs:complexType>
  <xs:element name="yesNoChoice" type="YesNoChoice"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest
import sys

class TestTrac0204 (unittest.TestCase):
    if sys.version_info[:2] < (2, 7):
        def assertIsNone (self, v):
            self.assertEqual(None, v)
        def assertIsNotNone (self, v):
            self.assertNotEqual(None, v)

    def testCtor (self):
        instance = yesNoChoice()
        self.assertIsNone(instance.Yes)
        self.assertIsNone(instance.No)
        instance = yesNoChoice(Yes=True)
        self.assertIsNotNone(instance.Yes)
        self.assertIsNone(instance.No)
        instance = yesNoChoice(Yes=True, No=True)
        self.assertRaises(pyxb.UnprocessedElementContentError, instance.validateBinding)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="elt" type="xs:string"/>
  <xs:complexType name="empty"/>
  <xs:complexType name="simple">
    <xs:simpleContent>
      <xs:extension base="xs:string"/>
    </xs:simpleContent>
  </xs:complexType>
  <xs:complexType name="complex">
    <xs:sequence>
      <xs:element ref="elt"/>
    </xs:sequence>
  </xs:complexType>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0021 (unittest.TestCase):
    """Presence of a wildcard in a sequence model group causes other
    elements in that group to not be generated."""
    def testEmpty (self):
        instance = empty()
        self.assertRaises(pyxb.NotSimpleContentError, instance.value)
        self.assertRaises(pyxb.NotComplexContentError, instance.orderedContent)

    def testSimple (self):
        instance = simple("hi")
        self.assertEqual("hi", instance.value())
        self.assertRaises(pyxb.NotComplexContentError, instance.orderedContent)

    def testComplex (self):
        instance = complex("hi")
        self.assertRaises(pyxb.NotSimpleContentError, instance.value)
        elt = instance.orderedContent()[0]
        self.assertTrue(isinstance(elt.value, six.string_types))
        self.assertEqual("hi", elt.value)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element type="xs:int" name="eOne"/>
  <xs:element type="xs:int" name="eTwo"/>
  <xs:complexType name="tElt">
    <xs:sequence>
      <xs:element ref="eOne" minOccurs="0"/>
      <xs:any namespace="##any" minOccurs="0" maxOccurs="3" processContents="lax"/>
    </xs:sequence>
  </xs:complexType>
  <xs:element name="elt" type="tElt"/>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0182 (unittest.TestCase):
    def testOne (self):
        ia = eOne(1)
        ib = eTwo(2)
        content = [ ia, ib ]
        i = elt(*content)
        xmls = '<elt><eOne>1</eOne><eTwo>2</eTwo></elt>'
        xmld = xmls.encode('utf-8')
        self.assertEqual(i.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(i.eOne, ia)
        self.assertEqual(1, len(i.wildcardElements()))
        self.assertEqual(ib, i.wildcardElements()[0])
        self.assertEqual(i.orderedContent()[0].value, ia)
        self.assertEqual(i.orderedContent()[1].value, ib)

if __name__ == '__main__':
    unittest.main()



# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import print_function
from pyxb.bundles.wssplat.raw.wsdl11 import *
import pyxb.bundles.wssplat.raw.wsdl11 as raw_wsdl11

import pyxb.namespace
from pyxb.utils import domutils, six
import xml.dom

def ImportRelatedNamespaces ():
    """Import modules for related namespaces so they are available to
    create binding instances from the WSDL sources."""
    try:
        import pyxb.bundles.wssplat.soapbind11
    except ImportError:
        pass
    try:
        import pyxb.bundles.wssplat.soapbind12
    except ImportError:
        pass
    try:
        import pyxb.bundles.wssplat.soap11
    except ImportError:
        pass
    try:
        import pyxb.bundles.wssplat.soap12
    except ImportError:
        pass
    try:
        import pyxb.bundles.wssplat.soapenv
    except ImportError:
        pass
    try:
        import pyxb.bundles.wssplat.httpbind
    except ImportError:
        pass
    try:
        import pyxb.bundles.wssplat.mimebind
    except ImportError:
        pass


class _WSDL_binding_mixin (object):
    """Mix-in class to mark element Python bindings that are expected
    to be wildcard matches in WSDL binding elements."""
    pass

class _WSDL_port_mixin (object):
    """Mix-in class to mark element Python bindings that are expected
    to be wildcard matches in WSDL port elements."""
    pass

class _WSDL_operation_mixin (object):
    """Mix-in class to mark element Python bindings that are expected
    to be wildcard matches in WSDL (binding) operation elements."""
    pass

class tPort (raw_wsdl11.tPort):
    def __getBindingReference (self):
        return self.__bindingReference
    def _setBindingReference (self, binding_reference):
        self.__bindingReference = binding_reference
    __bindingReference = None
    bindingReference = property(__getBindingReference)

    def __getAddressReference (self):
        return self.__addressReference
    def _setAddressReference (self, address_reference):
        self.__addressReference = address_reference
    __addressReference = None
    addressReference = property(__getAddressReference)

raw_wsdl11.tPort._SetSupersedingClass(tPort)

class tBinding (raw_wsdl11.tBinding):
    def __getPortTypeReference (self):
        return self.__portTypeReference
    def setPortTypeReference (self, port_type_reference):
        self.__portTypeReference = port_type_reference
    __portTypeReference = None
    portTypeReference = property(__getPortTypeReference)

    def __getProtocolBinding (self):
        """Return the protocol-specific binding information."""
        return self.__protocolBinding
    def _setProtocolBinding (self, protocol_binding):
        self.__protocolBinding = protocol_binding
    __protocolBinding = None
    protocolBinding = property(__getProtocolBinding)

    def operationMap (self):
        return self.__operationMap
    __operationMap = None

    def __init__ (self, *args, **kw):
        super(tBinding, self).__init__(*args, **kw)
        self.__operationMap = { }
raw_wsdl11.tBinding._SetSupersedingClass(tBinding)

class tPortType (raw_wsdl11.tPortType):
    def operationMap (self):
        return self.__operationMap
    __operationMap = None

    def __init__ (self, *args, **kw):
        super(tPortType, self).__init__(*args, **kw)
        self.__operationMap = { }
raw_wsdl11.tPortType._SetSupersedingClass(tPortType)

class tParam (raw_wsdl11.tParam):
    def __getMessageReference (self):
        return self.__messageReference
    def _setMessageReference (self, message_reference):
        self.__messageReference = message_reference
    __messageReference = None
    messageReference = property(__getMessageReference)
raw_wsdl11.tParam._SetSupersedingClass(tParam)

class tFault (raw_wsdl11.tFault):
    def __getMessageReference (self):
        return self.__messageReference
    def _setMessageReference (self, message_reference):
        self.__messageReference = message_reference
    __messageReference = None
    messageReference = property(__getMessageReference)
raw_wsdl11.tFault._SetSupersedingClass(tFault)

class tPart (raw_wsdl11.tPart):
    def __getElementReference (self):
        return self.__elementReference
    def _setElementReference (self, element_reference):
        self.__elementReference = element_reference
    __elementReference = None
    elementReference = property(__getElementReference)

    def __getTypeReference (self):
        return self.__typeReference
    def _setTypeReference (self, type_reference):
        self.__typeReference = type_reference
    __typeReference = None
    typeReference = property(__getTypeReference)
raw_wsdl11.tPart._SetSupersedingClass(tPart)

class tBindingOperation (raw_wsdl11.tBindingOperation):
    def __getOperationReference (self):
        return self.__operationReference
    def _setOperationReference (self, operation_reference):
        self.__operationReference = operation_reference
    __operationReference = None
    operationReference = property(__getOperationReference)
raw_wsdl11.tBindingOperation._SetSupersedingClass(tBindingOperation)

class tDefinitions (raw_wsdl11.tDefinitions):
    def messageMap (self):
        return self.targetNamespace().messages()

    def namespaceContext (self):
        return self.__namespaceContext
    __namespaceContext = None

    def bindingMap (self):
        return self.__bindingMap
    __bindingMap = None

    def targetNamespace (self):
        return self.namespaceContext().targetNamespace()

    def namespace (self):
        return self.__namespace
    __namespace = None

    def _addToMap (self, map, qname, value):
        map[qname] = value
        (ns, ln) = qname
        if (ns == self.targetNamespace()):
            map[(None, ln)] = value
        elif (ns is None):
            map[(self.targetNamespace(), ln)] = value
        return map

    def schema (self):
        return self.__schema
    __schema = None

    @classmethod
    def _PreFactory_vx (self, args, kw):
        # Import standard bindings.  If we do this, then wildcard
        # binding, port, and operation elements will be recognized and
        # converted into bindings.
        import pyxb.bundles.wssplat.soapbind11
        import pyxb.bundles.wssplat.soapbind12
        import pyxb.bundles.wssplat.httpbind

        # Ensure we have definitions for any externally-referenced
        # things we might need.  @todo: This might have to
        # chronologically precede the import above.
        pyxb.namespace.archive.NamespaceArchive.PreLoadArchives()

        raw_wsdl11.Namespace.validateComponentModel()
        state = ( kw.pop('process_schema', False),
                  kw.pop('generation_uid', None),
                  kw.get('_dom_node') )
        return state

    def _postFactory_vx (self, state):
        (process_schema, generation_uid, dom_node) = state
        assert isinstance(dom_node, xml.dom.Node)
        node_en = pyxb.namespace.ExpandedName(dom_node)
        self.__namespaceContext = pyxb.namespace.NamespaceContext.GetNodeContext(dom_node)
        self.__buildMaps()
        if process_schema:
            self.__processSchema(generation_uid)
        self.__finalizeReferences()
        return self

    __WSDLCategories = ( 'service', 'port', 'message', 'binding', 'portType' )
    def __buildMaps (self):
        tns = self.namespaceContext().targetNamespace()
        tns.configureCategories(self.__WSDLCategories)
        for m in self.message:
            tns.messages()[m.name] = m
        for pt in self.portType:
            tns.portTypes()[pt.name] = pt
            for op in pt.operation:
                pt.operationMap()[op.name] = op
                params = op.fault[:]
                if op.input is not None:
                    params.append(op.input)
                if op.output is not None:
                    params.append(op.output)
                for p in params:
                    msg_en = p.message
                    p._setMessageReference(p.message.message())
        for b in self.binding:
            tns.bindings()[b.name] = b
            port_type_en = b.type
            b.setPortTypeReference(port_type_en.portType())
            for wc in b.wildcardElements():
                if isinstance(wc, _WSDL_binding_mixin):
                    b._setProtocolBinding(wc)
                    break
            for op in b.operation:
                b.operationMap()[op.name] = op
                for wc in op.wildcardElements():
                    if isinstance(wc, _WSDL_operation_mixin):
                        op._setOperationReference(wc)
                        break
        for s in self.service:
            tns.services()[s.name] = s
            for p in s.port:
                binding_en = p.binding
                p._setBindingReference(binding_en.binding())
                for wc in p.wildcardElements():
                    if isinstance(wc, _WSDL_port_mixin):
                        p._setAddressReference(wc)
                        break

    def __processSchema (self, generation_uid):
        global pyxb
        import pyxb.xmlschema

        print('PS %s' % (generation_uid,))
        if self.__schema is not None:
            print('Already have schema')
            return self.__schema
        for t in self.types:
            for wc in t.wildcardElements():
                if isinstance(wc, xml.dom.Node) and pyxb.namespace.XMLSchema.nodeIsNamed(wc, 'schema'):
                    # Try to load component models for any namespace referenced by this.
                    # Probably shouldn't need to do this except for imported ones.
                    for ns in six.itervalues(self.namespaceContext().inScopeNamespaces()):
                        try:
                            ns.validateComponentModel()
                        except Exception as e:
                            print('Error validating component model for %s: %s' % (ns.uri(), e))
                    self.__schema = pyxb.xmlschema.schema.CreateFromDOM(wc, namespace_context=self.namespaceContext(), generation_uid=generation_uid)
                elif isinstance(wc, pyxb.xmlschema.schema):
                    self.__schema = wc
                else:
                    print('No match: %s %s' % (wc.namespaceURI, namespace.localName))
                if self.__schema is not None:
                    return self.__schema
        return None

    def __finalizeReferences (self):
        tns = self.namespaceContext().targetNamespace()
        for m in six.itervalues(tns.messages()):
            for p in m.part:
                if (p.element is not None) and (p.elementReference is None):
                    elt_en = p.element
                    p._setElementReference(elt_en.elementDeclaration())
                if (p.type is not None) and (p.typeReference is None):
                    type_en = p.type
                    p._setTypeReference(type_en.typeDefinition())

raw_wsdl11.tDefinitions._SetSupersedingClass(tDefinitions)

pyxb.namespace.NamespaceContext._AddTargetNamespaceAttribute(raw_wsdl11.Namespace.createExpandedName('definitions'), pyxb.namespace.ExpandedName('targetNamespace'))



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Classes supporting U{XMLSchema Part 2: Datatypes<http://www.w3.org/TR/xmlschema-2/>}.

Each L{simple type definition<pyxb.xmlschema.structures.SimpleTypeDefinition>} component
instance is paired with at most one L{basis.simpleTypeDefinition}
class, which is a subclass of a Python type augmented with facets and
other constraining information.  This file contains the definitions of
these types.

We want the simple datatypes to be efficient Python values, but to
also hold specific constraints that don't apply to the Python types.
To do this, we subclass each PST.  Primitive PSTs inherit from the
Python type that represents them, and from a
pyxb.binding.basis.simpleTypeDefinition class which adds in the
constraint infrastructure.  Derived PSTs inherit from the parent PST.

There is an exception to this when the Python type best suited for a
derived SimpleTypeDefinition differs from the type associated with its
parent STD: for example, L{xsd:integer<integer>} has a value range
that requires it be represented by a Python C{long}, but
L{xsd:int<int>} allows representation by a Python C{int}.  In this
case, the derived PST class is structured like a primitive type, but
the PST associated with the STD superclass is recorded in a class
variable C{_XsdBaseType}.

Note the strict terminology: "datatype" refers to a class which is a
subclass of a Python type, while "type definition" refers to an
instance of either SimpleTypeDefinition or ComplexTypeDefinition.

"""

import logging
import re
import binascii
import base64
import math
import decimal as python_decimal
from pyxb.exceptions_ import *
import pyxb.namespace
import pyxb.utils.unicode
from pyxb.utils import six
from . import basis

_log = logging.getLogger(__name__)

_PrimitiveDatatypes = []
_DerivedDatatypes = []
_ListDatatypes = []

# We use unicode as the Python type for anything that isn't a normal
# primitive type.  Presumably, only enumeration and pattern facets
# will be applied.
class anySimpleType (basis.simpleTypeDefinition, six.text_type):
    """XMLSchema datatype U{anySimpleType<http://www.w3.org/TR/xmlschema-2/#dt-anySimpleType>}."""
    _XsdBaseType = None
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('anySimpleType')

    @classmethod
    def XsdLiteral (cls, value):
        return value
# anySimpleType is not treated as a primitive, because its variety
# must be absent (not atomic).

class string (basis.simpleTypeDefinition, six.text_type):
    """XMLSchema datatype U{string<http://www.w3.org/TR/xmlschema-2/#string>}."""
    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('string')

    @classmethod
    def XsdLiteral (cls, value):
        assert isinstance(value, cls)
        return value

    @classmethod
    def XsdValueLength (cls, value):
        return len(value)

_PrimitiveDatatypes.append(string)

# It is illegal to subclass the bool type in Python, so we subclass
# int instead.
@six.python_2_unicode_compatible
class boolean (basis.simpleTypeDefinition, six.int_type):
    """XMLSchema datatype U{boolean<http://www.w3.org/TR/xmlschema-2/#boolean>}."""
    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('boolean')

    @classmethod
    def XsdLiteral (cls, value):
        if value:
            return 'true'
        return 'false'

    def __str__ (self):
        if self:
            return six.u('true')
        return six.u('false')

    def __new__ (cls, *args, **kw):
        args = cls._ConvertArguments(args, kw)
        if 0 < len(args):
            value = args[0]
            args = args[1:]
            if value in (1, 0, '1', '0', 'true', 'false'):
                if value in (1, '1', 'true'):
                    iv = True
                else:
                    iv = False
                return super(boolean, cls).__new__(cls, iv, *args, **kw)
            raise SimpleTypeValueError(cls, value)
        return super(boolean, cls).__new__(cls, *args, **kw)

_PrimitiveDatatypes.append(boolean)

class decimal (basis.simpleTypeDefinition, python_decimal.Decimal, basis._RepresentAsXsdLiteral_mixin):
    """XMLSchema datatype U{decimal<http://www.w3.org/TR/xmlschema-2/#decimal>}.

    This class uses Python's L{decimal.Decimal} class to support (by
    default) 28 significant digits.  Only normal and zero values are
    valid; this means C{NaN} and C{Infinity} may be created during
    calculations, but cannot be expressed in XML documents.
    """
    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('decimal')

    def __new__ (cls, *args, **kw):
        args = cls._ConvertArguments(args, kw)
        # Pre Python 2.7 can't construct from float values
        if (1 <= len(args)) and isinstance(args[0], six.float_type):
            args = (str(args[0]),) + args[1:]
        try:
            rv = super(decimal, cls).__new__(cls, *args, **kw)
        except python_decimal.DecimalException:
            raise SimpleTypeValueError(cls, *args)
        cls._CheckValidValue(rv)
        return rv

    @classmethod
    def _CheckValidValue (cls, value):
        if not (value.is_normal() or value.is_zero()):
            raise SimpleTypeValueError(cls, value)
        return super(decimal, cls)._CheckValidValue(value)

    @classmethod
    def XsdLiteral (cls, value):
        (sign, digits, exponent) = value.normalize().as_tuple()
        if (0 < len(digits)) and (0 == digits[0]):
            digits = ()
        rchars = []
        if sign:
            rchars.append('-')
        digits_before = len(digits) + exponent
        if 0 < digits_before:
            rchars.extend(map(str, digits[:digits_before]))
            digits = digits[digits_before:]
            if (0 == len(digits)) and (0 < exponent):
                rchars.extend(['0'] * exponent)
                exponent = 0
        else:
            rchars.append('0')
        rchars.append('.')
        digits_after = -exponent
        assert(0 <= digits_after)
        if 0 < digits_after:
            rchars.extend(['0'] * (digits_after - len(digits)))
            rchars.extend(map(str, digits))
        else:
            rchars.append('0')
        return six.u('').join(rchars)

_PrimitiveDatatypes.append(decimal)

class _fp (basis.simpleTypeDefinition, six.float_type):
    _XsdBaseType = anySimpleType

    @classmethod
    def XsdLiteral (cls, value):
        if math.isinf(value):
            if (0 > value):
                return '-INF'
            return 'INF'
        if math.isnan(value):
            return 'NaN'
        return '%s' % (value,)

class float (_fp):
    """XMLSchema datatype U{float<http://www.w3.org/TR/xmlschema-2/#float>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('float')

_PrimitiveDatatypes.append(float)

class double (_fp):
    """XMLSchema datatype U{double<http://www.w3.org/TR/xmlschema-2/#double>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('double')

_PrimitiveDatatypes.append(double)

import datetime

class duration (basis.simpleTypeDefinition, datetime.timedelta, basis._RepresentAsXsdLiteral_mixin):
    """XMLSchema datatype U{duration<http://www.w3.org/TR/xmlschema-2/#duration>}.

    This class uses the Python C{datetime.timedelta} class as its
    underlying representation.  This works fine as long as no months
    or years are involved, and no negative durations are involved.
    Because the XML Schema value space is so much larger, it is kept
    distinct from the Python value space, which reduces to integral
    days, seconds, and microseconds.

    In other words, the implementation of this type is a little
    shakey.

    """

    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('duration')

    __Lexical_re = re.compile('^(?P<neg>-?)P((?P<years>\d+)Y)?((?P<months>\d+)M)?((?P<days>\d+)D)?(?P<Time>T((?P<hours>\d+)H)?((?P<minutes>\d+)M)?(((?P<seconds>\d+)(?P<fracsec>\.\d+)?)S)?)?$')

    # We do not use weeks
    __XSDFields = ( 'years', 'months', 'days', 'hours', 'minutes', 'seconds' )
    __PythonFields = ( 'days', 'seconds', 'microseconds', 'minutes', 'hours' )

    def negativeDuration (self):
        return self.__negativeDuration
    __negativeDuration = None

    def durationData (self):
        return self.__durationData
    __durationData = None

    def __new__ (cls, *args, **kw):
        args = cls._ConvertArguments(args, kw)
        have_kw_update = False
        if not kw.get('_nil'):
            if 0 == len(args):
                raise SimpleTypeValueError(cls, args)
            text = args[0]
        if kw.get('_nil'):
            data = dict(zip(cls.__PythonFields, len(cls.__PythonFields) * [0,]))
            negative_duration = False
        elif isinstance(text, six.string_types):
            match = cls.__Lexical_re.match(text)
            if match is None:
                raise SimpleTypeValueError(cls, text)
            match_map = match.groupdict()
            if 'T' == match_map.get('Time'):
                # Can't have T without additional time information
                raise SimpleTypeValueError(cls, text)

            negative_duration = ('-' == match_map.get('neg'))

            fractional_seconds = 0.0
            if match_map.get('fracsec') is not None:
                fractional_seconds = six.float_type('0%s' % (match_map['fracsec'],))
                usec = six.int_type(1000000 * fractional_seconds)
                if negative_duration:
                    kw['microseconds'] = - usec
                else:
                    kw['microseconds'] = usec
            else:
                # Discard any bogosity passed in by the caller
                kw.pop('microsecond', None)

            data = { }
            for fn in cls.__XSDFields:
                v = match_map.get(fn, 0)
                if v is None:
                    v = 0
                data[fn] = six.int_type(v)
                if fn in cls.__PythonFields:
                    if negative_duration:
                        kw[fn] = - data[fn]
                    else:
                        kw[fn] = data[fn]
            data['seconds'] += fractional_seconds
            have_kw_update = True
        elif isinstance(text, cls):
            data = text.durationData().copy()
            negative_duration = text.negativeDuration()
        elif isinstance(text, datetime.timedelta):
            data = { 'days' : text.days,
                     'seconds' : text.seconds + (text.microseconds / 1000000.0) }
            negative_duration = (0 > data['days'])
            if negative_duration:
                if 0.0 == data['seconds']:
                    data['days'] = - data['days']
                else:
                    data['days'] = 1 - data['days']
                    data['seconds'] = 24 * 60 * 60.0 - data['seconds']
            data['minutes'] = 0
            data['hours'] = 0
        elif isinstance(text, six.integer_types) and (1 < len(args)):
            # Apply the arguments as in the underlying Python constructor
            data = dict(zip(cls.__PythonFields[:len(args)], args))
            negative_duration = False
        else:
            raise SimpleTypeValueError(cls, text)
        if not have_kw_update:
            rem_time = data.pop('seconds', 0)
            if (0 != (rem_time % 1)):
                data['microseconds'] = data.pop('microseconds', 0) + six.int_type(1000000 * (rem_time % 1))
                rem_time = rem_time // 1
            data['seconds'] = rem_time % 60
            rem_time = data.pop('minutes', 0) + (rem_time // 60)
            data['minutes'] = rem_time % 60
            rem_time = data.pop('hours', 0) + (rem_time // 60)
            data['hours'] = rem_time % 24
            data['days'] += (rem_time // 24)
            for fn in cls.__PythonFields:
                if fn in data:
                    if negative_duration:
                        kw[fn] = - data[fn]
                    else:
                        kw[fn] = data[fn]
                else:
                    kw.pop(fn, None)
            kw['microseconds'] = data.pop('microseconds', 0)
            data['seconds'] += kw['microseconds'] / 1000000.0

        rv = super(duration, cls).__new__(cls, **kw)
        rv.__durationData = data
        rv.__negativeDuration = negative_duration
        return rv

    @classmethod
    def XsdLiteral (cls, value):
        elts = []
        if value.negativeDuration():
            elts.append('-')
        elts.append('P')
        for k in ( 'years', 'months', 'days' ):
            v = value.__durationData.get(k, 0)
            if 0 != v:
                elts.append('%d%s' % (v, k[0].upper()))
        time_elts = []
        for k in ( 'hours', 'minutes' ):
            v = value.__durationData.get(k, 0)
            if 0 != v:
                time_elts.append('%d%s' % (v, k[0].upper()))
        v = value.__durationData.get('seconds', 0)
        if 0 != v:
            time_elts.append('%gS' % (v,))
        if 0 < len(time_elts):
            elts.append('T')
            elts.extend(time_elts)
        if 1 == len(elts):
            # Value must have zero duration.  Pick something short.
            elts.append('0D')
        return ''.join(elts)

_PrimitiveDatatypes.append(duration)

class _PyXBDateTime_base (basis.simpleTypeDefinition, basis._RepresentAsXsdLiteral_mixin):

    _Lexical_fmt = None
    """Format for the lexical representation of a date-related instance, excluding timezone.

    Subclasses must define this."""

    # Map from strptime/strftime formats to the regular expressions we
    # use to extract them.  We're more strict than strptime, so not
    # trying to use that.
    __PatternMap = { '%Y' : '(?P<negYear>-?)(?P<year>\d{4,})'
                   , '%m' : '(?P<month>\d{2})'
                   , '%d' : '(?P<day>\d{2})'
                   , '%H' : '(?P<hour>\d{2})'
                   , '%M' : '(?P<minute>\d{2})'
                   , '%S' : '(?P<second>\d{2})(?P<fracsec>\.\d+)?'
                   , '%Z' : '(?P<tzinfo>Z|[-+]\d\d:\d\d)' }

    # Cache of compiled regular expressions to parse lexical space of
    # a subclass.
    __LexicalREMap = { }

    # Fields extracted by parsing that have an integer value
    __LexicalIntegerFields = ( 'year', 'month', 'day', 'hour', 'minute', 'second' )

    _UTCTimeZone = pyxb.utils.utility.UTCOffsetTimeZone(0)
    """A L{datetime.tzinfo} instance representing UTC."""

    _LocalTimeZone = pyxb.utils.utility.LocalTimeZone()
    """A L{datetime.tzinfo} instance representing the local time zone."""

    _DefaultYear = 1900
    _DefaultMonth = 1
    _DefaultDay = 1

    @classmethod
    def _LexicalToKeywords (cls, text):
        lexical_re = cls.__LexicalREMap.get(cls)
        if lexical_re is None:
            pattern = '^' + cls._Lexical_fmt + '%Z?$'
            for (k, v) in six.iteritems(cls.__PatternMap):
                pattern = pattern.replace(k, v)
            lexical_re = re.compile(pattern)
            cls.__LexicalREMap[cls] = lexical_re
        match = lexical_re.match(text)
        if match is None:
            raise SimpleTypeValueError(cls, text)
        match_map = match.groupdict()
        kw = { }
        for (k, v) in six.iteritems(match_map):
            if (k in cls.__LexicalIntegerFields) and (v is not None):
                kw[k] = six.int_type(v)
        if '-' == match_map.get('negYear'):
            kw['year'] = - kw['year']
        if match_map.get('fracsec') is not None:
            kw['microsecond'] = six.int_type(round(1000000 * six.float_type('0%s' % (match_map['fracsec'],))))
        else:
            # Discard any bogosity passed in by the caller
            kw.pop('microsecond', None)
        if match_map.get('tzinfo') is not None:
            kw['tzinfo'] = pyxb.utils.utility.UTCOffsetTimeZone(match_map['tzinfo'])
        else:
            kw.pop('tzinfo', None)
        return kw

    @classmethod
    def _SetKeysFromPython_csc (cls, python_value, kw, fields):
        for f in fields:
            kw[f] = getattr(python_value, f)
        return getattr(super(_PyXBDateTime_base, cls), '_SetKeysFromPython_csc', lambda *a,**kw: None)(python_value, kw, fields)

    @classmethod
    def _SetKeysFromPython (cls, python_value, kw, fields):
        return cls._SetKeysFromPython_csc(python_value, kw, fields)

    # Several datetime classes are extension classes, and the PyXB
    # subclasses won't recognize the packed values.  Use the lexical
    # representation instead.
    def __reduce__ (self):
        return (self.__class__, (self.xsdLiteral(),))

    @classmethod
    def _AdjustForTimezone (cls, kw):
        """Update datetime keywords to account for timezone effects.

        All XML schema timezoned times are in UTC, with the time "in
        its timezone".  If the keywords indicate a non-UTC timezone is
        in force, and L{pyxb.PreserveInputTimeZone()} has not been
        set, adjust the values to account for the zone by subtracting
        the corresponding UTC offset and mark explicitly that the time
        is in UTC by leaving a C{tzinfo} attribute identifying the UTC
        time zone.

        @param kw: A dictionary of keywords relevant for a date or
        time instance.  The dictionary is updated by this call.
        """
        if pyxb.PreserveInputTimeZone():
            return
        tzoffs = kw.pop('tzinfo', None)
        if tzoffs is not None:
            use_kw = kw.copy()
            # Ensure ctor requirements of datetime.datetime are met
            use_kw.setdefault('year', cls._DefaultYear)
            use_kw.setdefault('month', cls._DefaultMonth)
            use_kw.setdefault('day', cls._DefaultDay)
            dt = datetime.datetime(tzinfo=tzoffs, **use_kw)
            dt -= tzoffs.utcoffset(dt)
            for k in six.iterkeys(kw):
                kw[k] = getattr(dt, k)
            kw['tzinfo'] = cls._UTCTimeZone

    @classmethod
    def XsdLiteral (cls, value):
        iso = value.replace(tzinfo=None).isoformat()
        if 0 <= iso.find('.'):
            iso = iso.rstrip('0')
        if value.tzinfo is not None:
            iso += value.tzinfo.tzname(value)
        return iso

class dateTime (_PyXBDateTime_base, datetime.datetime):
    """XMLSchema datatype U{dateTime<http://www.w3.org/TR/xmlschema-2/#dateTime>}.

    This class uses the Python C{datetime.datetime} class as its
    underlying representation.  Unless L{pyxb.PreserveInputTimeZone()}
    is used, all timezoned dateTime objects are in UTC.  Presence of
    time zone information in the lexical space is preserved by a
    non-empty tzinfo field, which should always be zero minutes offset
    from UTC unless the input time zone was preserved.

    @warning: The value space of Python's C{datetime.datetime} class
    is more restricted than that of C{xs:datetime}.  As a specific
    example, Python does not support negative years or years with more
    than four digits.  For now, the convenience of having an object
    that is compatible with Python is more important than supporting
    the full value space.  In the future, the choice may be left up to
    the developer.
    """

    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('dateTime')

    _Lexical_fmt = '%Y-%m-%dT%H:%M:%S'
    __CtorFields = ( 'year', 'month', 'day', 'hour', 'minute', 'second', 'microsecond', 'tzinfo' )

    def __new__ (cls, *args, **kw):
        args = cls._ConvertArguments(args, kw)

        ctor_kw = { }
        if kw.get('_nil'):
            ctor_kw = { 'year': 1900, 'month': 1, 'day': 1 }
        elif 1 == len(args):
            value = args[0]
            if isinstance(value, six.string_types):
                ctor_kw.update(cls._LexicalToKeywords(value))
            elif isinstance(value, datetime.datetime):
                cls._SetKeysFromPython(value, ctor_kw, cls.__CtorFields)
            elif isinstance(value, six.integer_types):
                raise TypeError('function takes at least 3 arguments (%d given)' % (len(args),))
            else:
                raise SimpleTypeValueError(cls, value)
        elif 3 <= len(args):
            for fi in range(len(cls.__CtorFields)):
                fn = cls.__CtorFields[fi]
                if fi < len(args):
                    ctor_kw[fn] = args[fi]
                elif fn in kw:
                    ctor_kw[fn] = kw[fn]
                kw.pop(fn, None)
        else:
            raise TypeError('function takes at least 3 arguments (%d given)' % (len(args),))

        cls._AdjustForTimezone(ctor_kw)
        kw.update(ctor_kw)
        year = kw.pop('year')
        month = kw.pop('month')
        day = kw.pop('day')
        rv = super(dateTime, cls).__new__(cls, year, month, day, **kw)
        return rv

    @classmethod
    def today (cls):
        """Return today.

        Just like datetime.datetime.today(), except this one sets a
        tzinfo field so it's clear the value is UTC."""
        return cls(datetime.datetime.now(cls._UTCTimeZone))

    def aslocal (self):
        """Returns a C{datetime.datetime} instance denoting the same
        time as this instance but adjusted to be in the local time
        zone.

        @rtype: C{datetime.datetime} (B{NOT} C{xsd.dateTime})
        """
        dt = self
        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=self._UTCTimeZone)
        return dt.astimezone(self._LocalTimeZone)

_PrimitiveDatatypes.append(dateTime)

class time (_PyXBDateTime_base, datetime.time):
    """XMLSchema datatype U{time<http://www.w3.org/TR/xmlschema-2/#time>}.

    This class uses the Python C{datetime.time} class as its
    underlying representation.  Note that per the XMLSchema spec, all
    dateTime objects are in UTC, and that timezone information in the
    string representation in XML is an indication of the local time
    zone's offset from UTC.  Presence of time zone information in the
    lexical space is indicated by the tzinfo field.

    @note: C{pyxb.PreserveInputTimeZone()} can be used to bypass the
    normalization to UTC.
    """

    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('time')

    _Lexical_fmt = '%H:%M:%S'
    __CtorFields = ( 'hour', 'minute', 'second', 'microsecond', 'tzinfo' )

    def __new__ (cls, *args, **kw):
        args = cls._ConvertArguments(args, kw)
        ctor_kw = { }
        if 1 <= len(args):
            value = args[0]
            if isinstance(value, six.string_types):
                ctor_kw.update(cls._LexicalToKeywords(value))
            elif isinstance(value, (datetime.time, datetime.datetime)):
                cls._SetKeysFromPython(value, ctor_kw, cls.__CtorFields)
            elif isinstance(value, six.integer_types):
                for fi in range(len(cls.__CtorFields)):
                    fn = cls.__CtorFields[fi]
                    if fi < len(args):
                        ctor_kw[fn] = args[fi]
                    elif fn in kw:
                        ctor_kw[fn] = kw[fn]
                    kw.pop(fn, None)
            else:
                raise SimpleTypeValueError(cls, value)

        cls._AdjustForTimezone(ctor_kw)
        kw.update(ctor_kw)
        return super(time, cls).__new__(cls, **kw)

_PrimitiveDatatypes.append(time)

class _PyXBDateOnly_base (_PyXBDateTime_base, datetime.datetime):
    _XsdBaseType = anySimpleType

    _ValidFields = ( 'year', 'month', 'day' )

    def __new__ (cls, *args, **kw):
        args = cls._ConvertArguments(args, kw)
        ctor_kw = { }
        ctor_kw['year'] = cls._DefaultYear
        ctor_kw['month'] = cls._DefaultMonth
        ctor_kw['day'] = cls._DefaultDay
        ctor_kw['hour'] = 0
        ctor_kw['minute'] = 0
        ctor_kw['second'] = 0
        if kw.get('_nil'):
            pass
        elif 1 <= len(args):
            value = args[0]
            if isinstance(value, six.string_types):
                if 1 != len(args):
                    raise TypeError('construction from string requires exactly 1 argument')
                ctor_kw.update(cls._LexicalToKeywords(value))
            elif isinstance(value, (datetime.date, datetime.datetime)):
                if 1 != len(args):
                    raise TypeError('construction from instance requires exactly 1 argument')
                cls._SetKeysFromPython(value, ctor_kw, cls._ValidFields)
                try:
                    tzinfo = value.tzinfo
                    if tzinfo is not None:
                        ctor_kw['tzinfo'] = tzinfo
                except AttributeError:
                    pass
            else:
                fi = 0
                while fi < len(cls._ValidFields):
                    fn = cls._ValidFields[fi]
                    if fi < len(args):
                        ctor_kw[fn] = args[fi]
                    elif fn in kw:
                        ctor_kw[fn] = kw[fn]
                    kw.pop(fn, None)
                    fi += 1
                if fi < len(args):
                    ctor_kw['tzinfo'] = args[fi]
                    fi += 1
                if fi != len(args):
                    raise TypeError('function takes %d arguments plus optional tzinfo (%d given)' % (len(cls._ValidFields), len(args)))
        else:
            raise TypeError('function takes %d arguments plus optional tzinfo' % (len(cls._ValidFields),))

        # Do not adjust for the timezone here.  Only xsd:date provides
        # a recoverable timezone, so just preserve the as-supplied
        # timezone, and we'll canonicalize the date one if/when it's
        # converted back to lexical form.
        kw.update(ctor_kw)
        argv = []
        argv.append(kw.pop('year'))
        argv.append(kw.pop('month'))
        argv.append(kw.pop('day'))
        return super(_PyXBDateOnly_base, cls).__new__(cls, *argv, **kw)

    @classmethod
    def XsdLiteral (cls, value):
        # Work around strftime year restriction
        fmt = cls._Lexical_fmt
        if value.year < 1900:
            fmt = fmt.replace('%Y', '%04d' % (value.year,))
            value = value.replace(year=1900)
        if value.tzinfo is not None:
            fmt += value.tzinfo.tzname(value)
        return value.strftime(fmt)

class date (_PyXBDateOnly_base):
    """XMLSchema datatype U{date<http://www.w3.org/TR/xmlschema-2/#date>}.

    This class uses the Python C{datetime.datetime} class as its
    underlying representation; fields not relevant to this type are
    derived from 1900-01-01T00:00:00.

    @note: Unlike L{dateTime}, timezoned date values are not converted
    to UTC.  The provided timezone information is retained along with
    the instance; however, the lexical representation generated for
    output is canonicalized (timezones no more than 12 hours off UTC).
    """

    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('date')
    _Lexical_fmt = '%Y-%m-%d'
    _Fields = ( 'year', 'month', 'day' )

    __SecondsPerMinute = 60
    __MinutesPerHalfDay = 12 * 60
    __MinutesPerDay = 24 * 60
    def xsdRecoverableTzinfo (self):
        """Return the recoverable tzinfo for the date.

        Return a L{pyxb.utils.utility.UTCOffsetTimeZone} instance
        reflecting the timezone associated with the date, or C{None}
        if the date is not timezoned.

        @note: This is not the recoverable timezone, because timezones are
        represented as timedeltas which get normalized in ways that
        don't match what we expect for a tzinfo.
        """
        if self.tzinfo is None:
            return None
        sdt = self.replace(hour=0, minute=0, second=0, tzinfo=self._UTCTimeZone)
        utc_offset = (sdt - self).seconds // self.__SecondsPerMinute
        if utc_offset > self.__MinutesPerHalfDay:
            utc_offset -= self.__MinutesPerDay
        return pyxb.utils.utility.UTCOffsetTimeZone(utc_offset)

    @classmethod
    def XsdLiteral (cls, value):
        # Work around strftime year restriction
        fmt = cls._Lexical_fmt
        rtz = value.xsdRecoverableTzinfo()
        if rtz is not None:
            # If the date is timezoned, convert it to UTC
            value -= value.tzinfo.utcoffset(value)
            value = value.replace(tzinfo=cls._UTCTimeZone)
        # Use the midpoint of the one-day interval to get the correct
        # month/day.
        value += datetime.timedelta(minutes=cls.__MinutesPerHalfDay)
        if value.year < 1900:
            fmt = fmt.replace('%Y', '%04d' % (value.year,))
            value = value.replace(year=1900)
        if rtz is not None:
            fmt += rtz.tzname(value)
        return value.strftime(fmt)

_PrimitiveDatatypes.append(date)

class gYearMonth (_PyXBDateOnly_base):
    """XMLSchema datatype U{gYearMonth<http://www.w3.org/TR/xmlschema-2/#gYearMonth>}.

    This class uses the Python C{datetime.datetime} class as its
    underlying representation; fields not relevant to this type are
    derived from 1900-01-01T00:00:00.
    """
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('gYearMonth')
    _Lexical_fmt = '%Y-%m'
    _ValidFields = ( 'year', 'month' )

_PrimitiveDatatypes.append(gYearMonth)

class gYear (_PyXBDateOnly_base):
    """XMLSchema datatype U{gYear<http://www.w3.org/TR/xmlschema-2/#gYear>}.

    This class uses the Python C{datetime.datetime} class as its
    underlying representation; fields not relevant to this type are
    derived from 1900-01-01T00:00:00.
    """
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('gYear')
    _Lexical_fmt = '%Y'
    _ValidFields = ( 'year', )
_PrimitiveDatatypes.append(gYear)

class gMonthDay (_PyXBDateOnly_base):
    """XMLSchema datatype U{gMonthDay<http://www.w3.org/TR/xmlschema-2/#gMonthDay>}.

    This class uses the Python C{datetime.datetime} class as its
    underlying representation; fields not relevant to this type are
    derived from 1900-01-01T00:00:00.
    """
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('gMonthDay')
    _Lexical_fmt = '--%m-%d'
    _ValidFields = ( 'month', 'day' )
_PrimitiveDatatypes.append(gMonthDay)

class gDay (_PyXBDateOnly_base):
    """XMLSchema datatype U{gDay<http://www.w3.org/TR/xmlschema-2/#gDay>}.

    This class uses the Python C{datetime.datetime} class as its
    underlying representation; fields not relevant to this type are
    derived from 1900-01-01T00:00:00.
    """
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('gDay')
    _Lexical_fmt = '---%d'
    _ValidFields = ( 'day', )
_PrimitiveDatatypes.append(gDay)

class gMonth (_PyXBDateOnly_base):
    """XMLSchema datatype U{gMonth<http://www.w3.org/TR/xmlschema-2/#gMonth>}.

    This class uses the Python C{datetime.datetime} class as its
    underlying representation; fields not relevant to this type are
    derived from 1900-01-01T00:00:00.
    """
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('gMonth')
    _Lexical_fmt = '--%m'
    _ValidFields = ( 'month', )
_PrimitiveDatatypes.append(gMonth)

class hexBinary (basis.simpleTypeDefinition, six.binary_type):
    """XMLSchema datatype U{hexBinary<http://www.w3.org/TR/xmlschema-2/#hexBinary>}."""
    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('hexBinary')

    @classmethod
    def _ConvertArguments_vx (cls, args, kw):
        if (1 <= len(args)) and kw.get('_from_xml', False):
            xmlt = args[0]
            try:
                xmld = xmlt.encode('utf-8')
                arg0 = binascii.unhexlify(xmld)
                args = (arg0,) + args[1:]
            except (TypeError, binascii.Error):
                raise SimpleTypeValueError(cls, args[0])
        return args

    @classmethod
    def XsdLiteral (cls, value):
        if isinstance(value, six.text_type):
            value = value.encode('utf-8')
        rvd = binascii.hexlify(value)
        rvt = rvd.decode('utf-8')
        return rvt.upper()

    @classmethod
    def XsdValueLength (cls, value):
        return len(value)

_PrimitiveDatatypes.append(hexBinary)

class base64Binary (basis.simpleTypeDefinition, six.binary_type):
    """XMLSchema datatype U{base64Binary<http://www.w3.org/TR/xmlschema-2/#base64Binary>}.

    See also U{RFC2045<http://tools.ietf.org/html/rfc2045>} and U{RFC4648<http://tools.ietf.org/html/rfc4648>}.
    """
    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('base64Binary')

    # base64 is too lenient: it accepts 'ZZZ=' as an encoding of
    # 'e\x96', while the required XML Schema production requires
    # 'ZZY='.  Define a regular expression per section 3.2.16.

    _B04 = '[AQgw]'
    _B04S = '(%s ?)' % (_B04,)
    _B16 = '[AEIMQUYcgkosw048]'
    _B16S = '(%s ?)' % (_B16,)
    _B64 = '[ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/]'
    _B64S = '(%s ?)' % (_B64,)

    __Pattern = '^((' + _B64S + '{4})*((' + _B64S + '{3}' + _B64 + ')|(' + _B64S + '{2}' + _B16S + '=)|(' + _B64S + _B04S + '= ?=)))?$'
    __Lexical_re = re.compile(__Pattern)

    __ValidateLength = None

    @classmethod
    def XsdValidateLength (cls, length):
        """Control the maximum encoded size that is checked for XML literal validity.

        Python's base64 module allows some literals that are invalid
        according to XML rules.  PyXB verifies the validity using a
        regular expression, which is costly for something that is
        unlikely to occur.  Use this function to inhibit checks for
        validity based on the length of the XML literal.

        @param length: C{None} (default) to check all literals,
        otherwise the maximum length literal that will be checked.
        Pass C{-1} to disable the validity check.

        @return: the previous validation length

        """
        rv = cls.__ValidateLength
        if (length is None) or isinstance(length, six.integer_types):
            cls.__ValidateLength = length
            return rv
        raise TypeError('must provide None or integer length')

    @classmethod
    def _ConvertArguments_vx (cls, args, kw):
        if (1 <= len(args)) and kw.get('_from_xml', False):
            xmlt = args[0]
            try:
                xmld = xmlt.encode('utf-8')
                arg0 = base64.standard_b64decode(xmld)
                args = (arg0,) + args[1:]
            except (TypeError, binascii.Error):
                raise SimpleTypeValueError(cls, xmlt)
            if (cls.__ValidateLength is None) or (cls.__ValidateLength >= len(xmlt)):
                # This is what it costs to try to be a validating processor.
                if cls.__Lexical_re.match(xmlt) is None:
                    raise SimpleTypeValueError(cls, xmlt)
        return args

    @classmethod
    def XsdLiteral (cls, value):
        if isinstance(value, six.text_type):
            value = value.encode('utf-8')
        rvd = base64.standard_b64encode(value)
        rvt = rvd.decode('utf-8')
        return rvt

    @classmethod
    def XsdValueLength (cls, value):
        return len(value)

_PrimitiveDatatypes.append(base64Binary)

class anyURI (basis.simpleTypeDefinition, six.text_type):
    """XMLSchema datatype U{anyURI<http://www.w3.org/TR/xmlschema-2/#anyURI>}."""
    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('anyURI')

    @classmethod
    def XsdValueLength (cls, value):
        return len(value)

    @classmethod
    def XsdLiteral (cls, value):
        return six.text_type(value)

_PrimitiveDatatypes.append(anyURI)

class QName (basis.simpleTypeDefinition, pyxb.namespace.ExpandedName):
    """XMLSchema datatype U{QName<http://www.w3.org/TR/xmlschema-2/#QName>}."""
    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('QName')

    @classmethod
    def XsdValueLength (cls, value):
        """Section 4.3.1.3: Legacy length return None to indicate no check"""
        return None

    @classmethod
    def _ConvertIf (cls, value, xmlns_context):
        if isinstance(value, pyxb.namespace.ExpandedName):
            assert 0 > value.localName().find(':')
            return value
        if not isinstance(value, six.string_types):
            raise SimpleTypeValueError(cls, value)
        if 0 <= value.find(':'):
            (prefix, local) = value.split(':', 1)
            if (NCName._ValidRE.match(prefix) is None) or (NCName._ValidRE.match(local) is None):
                raise SimpleTypeValueError(cls, value)
            if xmlns_context is None:
                raise pyxb.QNameResolutionError('QName resolution requires namespace context', value, xmlns_context)
            return xmlns_context.interpretQName(value, default_no_namespace=True)
        if NCName._ValidRE.match(value) is None:
            raise SimpleTypeValueError(cls, value)
        if xmlns_context is not None:
            return xmlns_context.interpretQName(value, default_no_namespace=True)
        return pyxb.namespace.ExpandedName(value)

    @classmethod
    def _ConvertArguments_vx (cls, args, kw):
        if 1 == len(args):
            xmlns_context = kw.pop('_xmlns_context', pyxb.namespace.NamespaceContext.Current())
            args = (cls._ConvertIf(args[0], xmlns_context),)
        super_fn = getattr(super(QName, cls), '_ConvertArguments_vx', lambda *a,**kw: args)
        return super_fn(args, kw)

    @classmethod
    def XsdLiteral (cls, value):
        # A QName has no unicode/XSD representation in the absence of
        # a registered namespace.  Whatever called this should have
        # detected that the value is a QName and used
        # BindingDOMSupport.qnameToText() to convert it to a lexical
        # representation that incorporates a declared namespace.
        raise pyxb.UsageError('Cannot represent QName without namespace declaration')

    @classmethod
    def _XsdConstraintsPreCheck_vb (cls, value):
        super_fn = getattr(super(QName, cls), '_XsdConstraintsPreCheck_vb', lambda *a,**kw: True)
        return super_fn(cls._ConvertIf(value, pyxb.namespace.NamespaceContext.Current()))


_PrimitiveDatatypes.append(QName)

class NOTATION (basis.simpleTypeDefinition):
    """XMLSchema datatype U{NOTATION<http://www.w3.org/TR/xmlschema-2/#NOTATION>}."""
    _XsdBaseType = anySimpleType
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('NOTATION')

    @classmethod
    def XsdValueLength (cls, value):
        """Section 4.3.1.3: Legacy length return None to indicate no check"""
        return None

_PrimitiveDatatypes.append(NOTATION)

class normalizedString (string):
    """XMLSchema datatype U{normalizedString<http:///www.w3.org/TR/xmlschema-2/#normalizedString>}.

    Normalized strings can't have carriage returns, linefeeds, or
    tabs in them."""

    # All descendents of normalizedString constrain the lexical/value
    # space in some way.  Subclasses should set the _ValidRE class
    # variable to a compiled regular expression that matches valid
    # input, or the _InvalidRE class variable to a compiled regular
    # expression that detects invalid inputs.
    #
    # Alternatively, subclasses can override the _ValidateString_va
    # method.

    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('normalizedString')

    # @todo Implement pattern constraints and just rely on them

    # No CR, LF, or TAB
    __BadChars = re.compile("[\r\n\t]")

    _ValidRE = None
    _InvalidRE = None

    @classmethod
    def __ValidateString (cls, value):
        # This regular expression doesn't work.  Don't know why.
        #if cls.__BadChars.match(value) is not None:
        #    raise SimpleTypeValueError('CR/NL/TAB characters illegal in %s' % (cls.__name__,))
        if (0 <= value.find("\n")) or (0 <= value.find("\r")) or (0 <= value.find("\t")):
            raise SimpleTypeValueError(cls, value)
        if cls._ValidRE is not None:
            match_object = cls._ValidRE.match(value)
            if match_object is None:
                raise SimpleTypeValueError(cls, value)
        if cls._InvalidRE is not None:
            match_object = cls._InvalidRE.match(value)
            if not (match_object is None):
                raise SimpleTypeValueError(cls, value)
        return True

    @classmethod
    def _ValidateString_va (cls, value):
        """Post-extended method to validate that a string matches a given pattern.

        If you can express the valid strings as a compiled regular
        expression in the class variable _ValidRE, or the invalid
        strings as a compiled regular expression in the class variable
        _InvalidRE, you can just use those.  If the acceptable matches
        are any trickier, you should invoke the superclass
        implementation, and if it returns True then perform additional
        tests."""
        super_fn = getattr(super(normalizedString, cls), '_ValidateString_va', lambda *a,**kw: True)
        if not super_fn(value):
            return False
        return cls.__ValidateString(value)

    @classmethod
    def _XsdConstraintsPreCheck_vb (cls, value):
        if not isinstance(value, six.string_types):
            raise SimpleTypeValueError(cls, value)
        if not cls._ValidateString_va(value):
            raise SimpleTypeValueError(cls, value)
        super_fn = getattr(super(normalizedString, cls), '_XsdConstraintsPreCheck_vb', lambda *a,**kw: True)
        return super_fn(value)

_DerivedDatatypes.append(normalizedString)
assert normalizedString.XsdSuperType() == string

class token (normalizedString):
    """XMLSchema datatype U{token<http:///www.w3.org/TR/xmlschema-2/#token>}.

    Tokens cannot leading or trailing space characters; any
    carriage return, line feed, or tab characters; nor any occurrence
    of two or more consecutive space characters."""

    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('token')

    @classmethod
    def _ValidateString_va (cls, value):
        super_fn = getattr(super(token, cls), '_ValidateString_va', lambda *a,**kw: True)
        if not super_fn(value):
            return False
        if value.startswith(" ") \
           or value.endswith(" ") \
           or (0 <= value.find('  ')):
            raise SimpleTypeValueError(cls, value)
        return True
_DerivedDatatypes.append(token)

class language (token):
    """XMLSchema datatype U{language<http:///www.w3.org/TR/xmlschema-2/#language>}"""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('language')
    _ValidRE = re.compile('^[a-zA-Z]{1,8}(-[a-zA-Z0-9]{1,8})*$')
_DerivedDatatypes.append(language)

class NMTOKEN (token):
    """XMLSchema datatype U{NMTOKEN<http:///www.w3.org/TR/xmlschema-2/#NMTOKEN>}.

    See U{http://www.w3.org/TR/2000/WD-xml-2e-20000814.html#NT-Nmtoken}.

    NMTOKEN is an identifier that can start with any character that is
    legal in it."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('NMTOKEN')
    _ValidRE = pyxb.utils.unicode.XML1p0e2.NmToken_re
_DerivedDatatypes.append(NMTOKEN)

class NMTOKENS (basis.STD_list):
    _ItemType = NMTOKEN
_ListDatatypes.append(NMTOKENS)

class Name (token):
    """XMLSchema datatype U{Name<http:///www.w3.org/TR/xmlschema-2/#Name>}.

    See U{http://www.w3.org/TR/2000/WD-xml-2e-20000814.html#NT-Name}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('Name')
    _ValidRE = pyxb.utils.unicode.XML1p0e2.Name_re
_DerivedDatatypes.append(Name)

class NCName (Name):
    """XMLSchema datatype U{NCName<http:///www.w3.org/TR/xmlschema-2/#NCName>}.

    See U{http://www.w3.org/TR/1999/REC-xml-names-19990114/#NT-NCName}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('NCName')
    _ValidRE = pyxb.utils.unicode.XML1p0e2.NCName_re
_DerivedDatatypes.append(NCName)

class ID (NCName):
    """XMLSchema datatype U{ID<http:///www.w3.org/TR/xmlschema-2/#ID>}."""
    # Lexical and value space match that of parent NCName
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('ID')
    pass
_DerivedDatatypes.append(ID)

class IDREF (NCName):
    """XMLSchema datatype U{IDREF<http:///www.w3.org/TR/xmlschema-2/#IDREF>}."""
    # Lexical and value space match that of parent NCName
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('IDREF')
    pass
_DerivedDatatypes.append(IDREF)

class IDREFS (basis.STD_list):
    """XMLSchema datatype U{IDREFS<http:///www.w3.org/TR/xmlschema-2/#IDREFS>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('IDREFS')
    _ItemType = IDREF
_ListDatatypes.append(IDREFS)

class ENTITY (NCName):
    """XMLSchema datatype U{ENTITY<http:///www.w3.org/TR/xmlschema-2/#ENTITY>}."""
    # Lexical and value space match that of parent NCName; we're gonna
    # ignore the additional requirement that it be declared as an
    # unparsed entity
    #
    # @todo Don't ignore the requirement that this be declared as an
    # unparsed entity.
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('ENTITY')
    pass
_DerivedDatatypes.append(ENTITY)

class ENTITIES (basis.STD_list):
    """XMLSchema datatype U{ENTITIES<http:///www.w3.org/TR/xmlschema-2/#ENTITIES>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('ENTITIES')
    _ItemType = ENTITY
_ListDatatypes.append(ENTITIES)

class integer (basis.simpleTypeDefinition, six.long_type):
    """XMLSchema datatype U{integer<http://www.w3.org/TR/xmlschema-2/#integer>}."""
    _XsdBaseType = decimal
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('integer')

    @classmethod
    def XsdLiteral (cls, value):
        return '%d' % (value,)

_DerivedDatatypes.append(integer)

class nonPositiveInteger (integer):
    """XMLSchema datatype U{nonPositiveInteger<http://www.w3.org/TR/xmlschema-2/#nonPositiveInteger>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('nonPositiveInteger')
_DerivedDatatypes.append(nonPositiveInteger)

class negativeInteger (nonPositiveInteger):
    """XMLSchema datatype U{negativeInteger<http://www.w3.org/TR/xmlschema-2/#negativeInteger>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('negativeInteger')
_DerivedDatatypes.append(negativeInteger)

class long (integer):
    """XMLSchema datatype U{long<http://www.w3.org/TR/xmlschema-2/#long>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('long')
_DerivedDatatypes.append(long)

class int (basis.simpleTypeDefinition, six.int_type):
    """XMLSchema datatype U{int<http://www.w3.org/TR/xmlschema-2/#int>}."""
    _XsdBaseType = long
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('int')

    @classmethod
    def XsdLiteral (cls, value):
        return '%s' % (value,)

    pass
_DerivedDatatypes.append(int)

class short (int):
    """XMLSchema datatype U{short<http://www.w3.org/TR/xmlschema-2/#short>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('short')
_DerivedDatatypes.append(short)

class byte (short):
    """XMLSchema datatype U{byte<http://www.w3.org/TR/xmlschema-2/#byte>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('byte')
_DerivedDatatypes.append(byte)

class nonNegativeInteger (integer):
    """XMLSchema datatype U{nonNegativeInteger<http://www.w3.org/TR/xmlschema-2/#nonNegativeInteger>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('nonNegativeInteger')
_DerivedDatatypes.append(nonNegativeInteger)

class unsignedLong (nonNegativeInteger):
    """XMLSchema datatype U{unsignedLong<http://www.w3.org/TR/xmlschema-2/#unsignedLong>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('unsignedLong')
_DerivedDatatypes.append(unsignedLong)

class unsignedInt (unsignedLong):
    """XMLSchema datatype U{unsignedInt<http://www.w3.org/TR/xmlschema-2/#unsignedInt>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('unsignedInt')
_DerivedDatatypes.append(unsignedInt)

class unsignedShort (unsignedInt):
    """XMLSchema datatype U{unsignedShort<http://www.w3.org/TR/xmlschema-2/#unsignedShort>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('unsignedShort')
_DerivedDatatypes.append(unsignedShort)

class unsignedByte (unsignedShort):
    """XMLSchema datatype U{unsignedByte<http://www.w3.org/TR/xmlschema-2/#unsignedByte>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('unsignedByte')
_DerivedDatatypes.append(unsignedByte)

class positiveInteger (nonNegativeInteger):
    """XMLSchema datatype U{positiveInteger<http://www.w3.org/TR/xmlschema-2/#positiveInteger>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('positiveInteger')
_DerivedDatatypes.append(positiveInteger)

from . import content

class anyType (basis.complexTypeDefinition):
    """XMLSchema datatype U{anyType<http://www.w3.org/TR/2001/REC-xmlschema-1-20010502/#key-urType>}."""
    _ExpandedName = pyxb.namespace.XMLSchema.createExpandedName('anyType')
    _DefinitionLocation = pyxb.utils.utility.Location('http://www.w3.org/TR/2001/REC-xmlschema-1-20010502/#key-urType', 1, 1)
    _ContentTypeTag = basis.complexTypeDefinition._CT_MIXED
    _Abstract = False
    _HasWildcardElement = True
    _AttributeWildcard = content.Wildcard(namespace_constraint=content.Wildcard.NC_any, process_contents=content.Wildcard.PC_lax)

def _BuildAutomaton ():
    # Remove this helper function from the namespace after it's invoked
    global _BuildAutomaton
    del _BuildAutomaton
    import pyxb.utils.fac as fac

    counters = set()
    cc_0 = fac.CounterCondition(min=0, max=None, metadata=pyxb.utils.utility.Location('http://www.w3.org/TR/2001/REC-xmlschema-1-20010502/#key-urType', 1, 1))
    counters.add(cc_0)
    states = set()
    final_update = set()
    final_update.add(fac.UpdateInstruction(cc_0, False))
    symbol = content.WildcardUse(content.Wildcard(process_contents=content.Wildcard.PC_lax, namespace_constraint=content.Wildcard.NC_any), None)
    st_0 = fac.State(symbol, is_initial=True, final_update=final_update, is_unordered_catenation=False)
    states.add(st_0)
    transitions = set()
    transitions.add(fac.Transition(st_0, [
        fac.UpdateInstruction(cc_0, True) ]))
    st_0._set_transitionSet(transitions)
    return fac.Automaton(states, counters, True, containing_state=None)
anyType._Automaton = _BuildAutomaton()


# anyType._IsUrType() is True; foo._IsUrType() for descendents of it
# should be false.
anyType._IsUrType = classmethod(lambda _c: _c == anyType)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-namespace-uu.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestNamespaceUU (unittest.TestCase):
    def setUp (self):
        # Prepare to hide warnings about failure to convert DOM node
        self.__basis_log = logging.getLogger('pyxb.binding.basis')
        self.__basis_loglevel = self.__basis_log.level

    def tearDown (self):
        self.__basis_log.level = self.__basis_loglevel

    def testBad (self):
        # Hide warnings about no binding for DOM nodes that are
        # uninteresting; we want to catch the validation errors.
        self.__basis_log.setLevel(logging.ERROR)
        # Default namespace improperly gives namespace to local element
        xml = '<globalStruct xmlns="urn:namespaceTest"><local>local</local><globalElt>global</globalElt></globalStruct>'
        self.assertRaises(pyxb.UnrecognizedContentError, CreateFromDocument, xml)
        # Did not add namespace to internal global element
        xml = '<ns1:globalStruct xmlns:ns1="urn:namespaceTest"><local>local</local><globalElt>global</globalElt></ns1:globalStruct>'
        self.assertRaises(pyxb.UnrecognizedContentError, CreateFromDocument, xml)
        # Improperly added namespace to internal local element
        xml = '<ns1:globalStruct xmlns:ns1="urn:namespaceTest"><ns1:local>local</ns1:local><ns1:globalElt>global</ns1:globalElt></ns1:globalStruct>'
        self.assertRaises(pyxb.UnrecognizedContentError, CreateFromDocument, xml)

    def testGood (self):
        xml = '<ns1:globalStruct xmlns:ns1="urn:namespaceTest"><local>local</local><ns1:globalElt>global</ns1:globalElt></ns1:globalStruct>'
        instance = CreateFromDocument(xml)
        self.assertEqual(instance.local, 'local')
        self.assertEqual(instance.globalElt, 'global')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.exceptions_ import *
import unittest
import pyxb.binding.datatypes as xsd

class _TestIntegerType (object):
    """Base class for testing any datatype that descends from integer.

    Subclasses should define class variables:
    THIS_TYPE = the xsd datatype class
    PARENT_TYPE = the next dominating type in the hierarchy
    MIN_IN_RANGE = the minimum expressible value
    MAX_IN_RANGE = the maximum expressible value

    Optional values to set:
    ZERO_IN_RANGE = False if zero not valid for subclass; default is True

    """

    MIN_IN_RANGE = None
    ZERO_IN_RANGE = True
    MAX_IN_RANGE = None

    def testParentage (self):
        self.assertTrue(self.PARENT_TYPE == self.THIS_TYPE.XsdSuperType())

    def testRange (self):
        if self.MIN_IN_RANGE is not None:
            if not ((self.MIN_IN_RANGE-1) in self.PARENT_EXCLUDE):
                self.assertRaises(SimpleTypeValueError, self.THIS_TYPE, self.MIN_IN_RANGE - 1)
            self.assertEqual(self.MIN_IN_RANGE, self.THIS_TYPE(self.MIN_IN_RANGE))
        if self.ZERO_IN_RANGE:
            self.assertEqual(0, self.THIS_TYPE(0))
        if self.MAX_IN_RANGE is not None:
            self.assertEqual(self.MAX_IN_RANGE, self.THIS_TYPE(self.MAX_IN_RANGE))
            if not ((self.MAX_IN_RANGE+1) in self.PARENT_EXCLUDE):
                self.assertRaises(SimpleTypeValueError, self.THIS_TYPE, self.MAX_IN_RANGE+1)

    PARENT_EXCLUDE = []

    def testStringConversion (self):
        numbers = [ ]
        if self.MIN_IN_RANGE is not None:
            numbers.extend([self.MIN_IN_RANGE-1, self.MIN_IN_RANGE])
        if self.ZERO_IN_RANGE:
            numbers.append(0)
        if self.MAX_IN_RANGE is not None:
            numbers.extend([self.MAX_IN_RANGE, self.MAX_IN_RANGE+1])
        for n in numbers:
            s = '%d' % (n,)
            p = None
            if not (n in self.PARENT_EXCLUDE):
                p = self.PARENT_TYPE(n)
                self.assertEqual(n, p)
            if ((self.MIN_IN_RANGE is None) or (self.MIN_IN_RANGE <= n)) \
               and ((self.MAX_IN_RANGE is None) or (n <= self.MAX_IN_RANGE)):
                bs = self.THIS_TYPE(s)
                self.assertEqual(n, bs)
                self.assertEqual(s, bs.xsdLiteral())
                bp = self.THIS_TYPE(p)
                self.assertEqual(n, bp)
            else:
                self.assertRaises(SimpleTypeValueError, self.THIS_TYPE, s)
                if p is not None:
                    self.assertRaises(SimpleTypeValueError, self.THIS_TYPE, p)

class Test_byte (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.byte
    PARENT_TYPE = xsd.short
    MIN_IN_RANGE = -128
    MAX_IN_RANGE = 127

class Test_unsignedByte (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.unsignedByte
    PARENT_TYPE = xsd.unsignedShort
    PARENT_EXCLUDE = [ -1 ]
    MIN_IN_RANGE = 0
    MAX_IN_RANGE = 255

class Test_short (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.short
    PARENT_TYPE = xsd.int
    MIN_IN_RANGE = -32768
    MAX_IN_RANGE = 32767

class Test_unsignedShort (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.unsignedShort
    PARENT_TYPE = xsd.unsignedInt
    PARENT_EXCLUDE = [ -1 ]
    MIN_IN_RANGE = 0
    MAX_IN_RANGE = 65535

class Test_int (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.int
    PARENT_TYPE = xsd.long
    MIN_IN_RANGE = -2147483648
    MAX_IN_RANGE = 2147483647

class Test_unsignedInt (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.unsignedInt
    PARENT_TYPE = xsd.unsignedLong
    PARENT_EXCLUDE = [ -1 ]
    MIN_IN_RANGE = 0
    MAX_IN_RANGE = 4294967295

class Test_long (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.long
    PARENT_TYPE = xsd.integer
    MIN_IN_RANGE = -9223372036854775808
    MAX_IN_RANGE = 9223372036854775807

class Test_unsignedLong (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.unsignedLong
    PARENT_TYPE = xsd.nonNegativeInteger
    PARENT_EXCLUDE = [ -1 ]
    MIN_IN_RANGE = 0
    MAX_IN_RANGE = 18446744073709551615

class Test_negativeInteger (unittest.TestCase, _TestIntegerType):
    ZERO_IN_RANGE = False
    THIS_TYPE = xsd.negativeInteger
    PARENT_TYPE = xsd.nonPositiveInteger
    MAX_IN_RANGE = -1

class Test_nonPositiveInteger (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.nonPositiveInteger
    PARENT_TYPE = xsd.integer
    MAX_IN_RANGE = 0

class Test_nonNegativeInteger (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.nonNegativeInteger
    PARENT_TYPE = xsd.integer
    MIN_IN_RANGE = 0

class Test_positiveInteger (unittest.TestCase, _TestIntegerType):
    THIS_TYPE = xsd.positiveInteger
    PARENT_TYPE = xsd.nonNegativeInteger
    MIN_IN_RANGE = 1
    ZERO_IN_RANGE = False

if __name__ == '__main__':
    unittest.main()
